#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Create Chain of productions for ILD.

:author: S. Poss, A.Sailer, C. Calancha
:since: Mar 26, 2012
"""

from __future__ import print_function
from __future__ import absolute_import
import pprint

# pylint: disable=invalid-name, wrong-import-position

from DIRAC.Core.Base import Script
Script.parseCommandLine()

from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations


from ILCDIRAC.Interfaces.API.NewInterface.ILDProductionJobDBD import ILDProductionJobDBD
from ILCDIRAC.Interfaces.API.NewInterface.Applications import Mokka, Marlin, OverlayInput
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLCIOSplit, StdHepSplit


####################################################################################
'''
Example to run ILDProductionChainDBD.py. This script creates 3 transformations
for stdhepsplit, Mokka simulation, and Marlin reconsturuction with Overlay.

Meta keys for input data query used by stdhepsplit are
Energy, Machine, GenProcessName, GenProcessID, MachineParams, SelectedFile

This script can take just one GenProcessID as an input.
Stdhep files which has the SelectedFile meta key equal to a selectedFile value
are taken as input.
Meta keys for input data query for Mokka and Marlin are generated automatically
by this script.

The number of tasks for stdhepsplit and Mokka simulation are limited to
1 and 5 respectively, using Limited prod plugin. Use a command,
  $ dirac-ilc-add-tasks-to-prod -p [prod_id] -t [number_of_tasks_to_add]
in order to create more tasks.

Output data are stored in
  splitted-stdhep : /ilc/prod/ilc/mc-dbd.generated/splitted
  sim, rec        : /ilc/prod/ilc/mc-dbd/ild/[sim|rec]
  dst             : /ilc/prod/ilc/mc-dbd.log/dst
Output SE can be specified separately.
This should be corrected properly for actual production.

SoftwareTag meta key value defined to existing directories are in consistent
in some directory, stopping this script to run if output to mc-dbd directory.
A wrong SoftwareTag key has to be removed in such case.

'''
#####################################################################################

dryrun = False
# do not register anything nor create anything.
# Should be used once the splitting-at-stdhep-level prods are submitted.
# TODO: add evttype to the ProdGroup

analysis = 'ILD-DBD-TEST'  # Some analysis: the prods will belong to the ProdGroup
my_evttype = 'higgs_ffh'
my_evtclass = 'higgs'
selectedfile = 1
prodid = 6556
genprocessname = 'qqh_ww_4q'
process = '106730'
energy = 500.  # This is mostly needed to define easily the steering files and the overlay parameters
analysis += '_' + my_evttype

meta_energy = str(int(energy))  # This is needed for the meta data search below

# for the overlay: using DBD numbers
BXOverlay = 1
GGToHadInt250 = 0.2
GGToHadInt350 = 0.33
GGToHadInt500 = 1.7
GGToHadInt1000 = 4.1

MarlinVer = "ILCSoft-01-16-02-p1"
ILDConfig = ''  # Set below for different energies
MokkaVer = "080003"
MokkaILDConfig = "v01-14-01-p00"
banned_sites = [""]

detectorModel = 'ILD_o1_v05'  # OR anything valid, but be careful with the overlay, the files need to exist
dbslice = "mokka-08-00-dbdump.sql"

machineParameters = 'TDR_ws'
if energy == 1000.:
  # ILDConfig         = 'v01-14-01-p00' # this is wrong!
  ILDConfig = ILDConfig or 'v01-16-p03'
  machineParameters = 'B1b_ws'
elif energy == 500.:
  ILDConfig = ILDConfig or 'v01-16-p05_500'
elif energy == 350.:
  ILDConfig = ILDConfig or 'v01-16-p09_350'
elif energy == 250.:
  ILDConfig = ILDConfig or 'v01-16-p10_250'
else:
  print("ILDConfig ILD: No ILDConfig defined for this energy (%.1f GeV)" % energy)

additional_name = '_' + genprocessname + '_DBD_2017030_1_' + str(selectedfile) + '_ildconfig-' + ILDConfig

energyMachinePars = meta_energy + '-' + machineParameters
# Following variables avoid output from stdhepsplit being used
# as input for the same production. Also speed up the job submission.

# Setting for code development
grand_base = '/ilc/prod/ilc/ild/test/temp1/'  # Test area for development
basepath = grand_base + 'mc-dbd/ild/'
diskpath = grand_base + 'mc-dbd.disk/ild/'  # This is not a disk directory now
dstPath = diskpath  # Basepath for dst output of marlin

''' possible setting for real production
grand_base = '/ilc/prod/ilc/'  # Test area for development
basepath = grand_base + 'mc-dbd/ild/'
diskpath = grand_base + 'mc-dbd.generated/ild/'
dstPath = grand_base + 'mc-dbd.log/ild/'  # Basepath for dst output of marlin
'''

matchToInput_stdhepsplit = '/ilc/prod/ilc/mc-dbd/generated/' + \
    energyMachinePars + '/' + my_evtclass  # Input for stdhepSplit
stdhepsplit_basepath = diskpath + 'splitted/'  # Base path for stdhepsplit output
matchToInput_mokka = stdhepsplit_basepath + energyMachinePars + '/' + my_evttype
matchToInput_marlin = basepath + "sim/" + energyMachinePars + \
    '/' + my_evttype + '/' + detectorModel + '/' + MokkaILDConfig
my_basepath = basepath  # my_basepath is a base path for Mokka (Marlin) output
recPath = basepath  # Basepath for rec output of marlin

# Define output SE depending on expected data size
SE_stdhepsplit = "PNNL-SRM"
SE_sim = "KEK-SRM"
SE_psplit = "PNNL-SRM"
SE_rec = "DESY-SRM"

print("###### Output file paths and SEs ########################################")
print("matchToInput_stdhepsplit (input)      =" + matchToInput_stdhepsplit)
print("stdhepsprint_basepath (output)        =" + stdhepsplit_basepath)
print("matchToInput_mokka (input)            =" + matchToInput_mokka)
print("matchToInput_marlin (input)           =" + matchToInput_marlin)
print("Basepath for sim&rec (output)         =" + basepath)
print("Diskpath for stdhepsplit&dst (output) =" + diskpath)
print("Outtput SE : stdhepsplit(%s), SUM(%s), psplit(%s), REC&DST(%s)" % (SE_stdhepsplit, SE_sim, SE_psplit, SE_rec))
print("#########################################################################")

###LCG_SITE  = "LCG.KEK.jp"
input_sand_box = [""]
# This is where magic happens
meta = {}

meta['Datatype'] = 'gen'  # MOKKA or stdhepsplit or MOKKA+MARLIN
# meta['Datatype']      = 'SIM' # JUST MARLIN / MARLIN_OVERLAY

meta['Energy'] = meta_energy
meta['Machine'] = 'ilc'
meta['GenProcessName'] = genprocessname
meta['MachineParams'] = machineParameters

# GenProcessID or ProcessID
if meta['Datatype'] == 'gen':
  meta['GenProcessID'] = process
else:
  meta['ProcessID'] = process
  # These parameters automatically retrieved if you run Mokka. If
  # running standalone Marlin you need to specify them
  meta['DetectorModel'] = detectorModel
  meta['EvtClass'] = my_evttype
  meta['MachineParams'] = machineParameters
  meta['ProdID'] = prodid

# DoSplit at stdhep level
activesplitstdhep = True
nbevtsperfilestdhep = 500  # Number of events per splitted-stdhep file = #Events for Sim and Rec jobs.
nbtasks_split = 1  # -1 to run over all input stdhep
if activesplitstdhep:
  if selectedfile > 0:
    meta['SelectedFile'] = selectedfile
  else:
    print('ERROR: stdhepsplit requires SelectedFile in the metadata to prevent output being used as input.')
    exit(1)
else:
  if prodid:
    meta['ProdID'] = prodid
  if selectedfile > 0:
    meta['SelectedFile'] = selectedfile
    print('Warning: SelectedFile meta field active: this should only happen when debugging.')

# Do Sim
ild_sim = True
nbtasks_sim = 5  # Take 5 files from input meta data query for job submission
# nbtasks_sim = -1 #Take all files from input meta data query result
# It can be extended with dirac-ilc-add-tasks-to-prod
# It's possible to get this number automatically by getting the number of events per file (if known)
# nbtasks = math.ceil(number_of_events_to_process/nb_events_per_signal_file) #needs import math

# DoSplit
activesplit = False
nbevtsperfile = 200

# Do Reco with Overlay
ild_rec_ov = True
nbtasks_rec_ov = 5  # See comment on nbtasks_sim
# Do Reco
ild_rec = False  # please, use WITH OVERLAY
nbtasks_rec = -1  # See comment on nbtasks_sim

# Whatever is below is not to be touched... Or at least only when something changes

# Split
stdhepsplit = StdHepSplit()
stdhepsplit.setVersion("V2")
stdhepsplit.setNumberOfEventsPerFile(nbevtsperfilestdhep)

# Simulation ILD
mo = Mokka()
mo.setVersion(MokkaVer)  # SET HERE YOUR MOKKA VERSION, the software will come from the ILDConfig
mo.setDetectorModel(detectorModel)
mo.setSteeringFile("bbudsc_3evt.steer")
# Do not include '.tgz'
mo.setDbSlice(dbslice)

# Split
split = SLCIOSplit()
split.setNumberOfEventsPerFile(nbevtsperfile)

# Define the overlay
overlay = OverlayInput()
overlay.setMachine("ilc_dbd")  # Don't touch, this is how the system knows what files to get
overlay.setEnergy(energy)  # Don't touch, this is how the system knows what files to get
overlay.setDetectorModel(detectorModel)  # Don't touch, this is how the system knows what files to get
if energy == 500.:  # here you chose the overlay parameters as this determines how many files you need
  # it does NOT affect the content of the marlin steering file whatsoever, you need to make sure the values
  # there are correct. Only the file names are handled properly so that you don't need to care
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt500)
  overlay.setBkgEvtType("aa_lowpt2")  # lowpt2: correct number of events (500),
  # not increased to 2500 to reduce number
  # of downloaded files
elif energy == 1000.:
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt1000)
  overlay.setBkgEvtType("aa_lowpt")
elif energy == 350.:
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt350)
  overlay.setBkgEvtType("aa_lowpt")
elif energy == 250.:
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt250)
  overlay.setBkgEvtType("aa_lowpt")

else:
  print("Overlay ILD: No overlay parameters defined for this energy")

# Reconstruction ILD with overlay
mao = Marlin()
mao.setDebug()
mao.setVersion(MarlinVer)  # PUT HERE YOUR MARLIN VERSION
if ild_rec_ov:
  if energy in [250.0, 350.0, 500.0, 1000.0]:
    mao.setSteeringFile("bbudsc_3evt_stdreco.xml")
    mao.setGearFile("GearOutput.xml")
  else:
    print("Marlin: No reconstruction suitable for this energy")


# Reconstruction ILD w/o overlay
ma = Marlin()
ma.setDebug()
ma.setVersion(MarlinVer)
ma.setEnergy(energy)
if ild_rec:
  if energy in [250.0, 350.0, 500.0, 1000.0]:
    ma.setSteeringFile("stdreco.xml")
    ma.setGearFile("GearOutput.xml")
  else:
    print("Marlin: No reconstruction suitable for this energy %g" % (energy))


###################################################################################
# HERE WE DEFINE THE PRODUCTIONS
if activesplitstdhep and meta:
  print("################## Createing a production for stdhepsplit: input meta is")
  pprint.pprint(meta)
  pstdhepsplit = ILDProductionJobDBD()
  pstdhepsplit.basepath = stdhepsplit_basepath  # stdhepsplit output directory
  pstdhepsplit.matchToInput = matchToInput_stdhepsplit
  pstdhepsplit.setDryRun(dryrun)
  # pstdhepsplit.setILDConfig(ILDConfig) ## stdhepsplit does not need ILDConfig
  pstdhepsplit.setEvtClass(my_evtclass)
  pstdhepsplit.setEvtType(my_evttype)
  # pstdhepsplit.setUseSoftTagInPath(False)
  pstdhepsplit.setLogLevel("verbose")
  pstdhepsplit.setProdType('Split')
  pstdhepsplit.setBannedSites(banned_sites)
  pstdhepsplit.setProdPlugin('Limited')  # exit with error: it seems i need
  # to set the Prod. plugin

  # generated files has not SoftwareTag: we exclude it from inputdataquery, but
  # reinserted at the end of the module to be used in the next simulation module
  tmp_softwaretag_val = ''
  if 'SoftwareTag' in meta:
    tmp_softwaretag_val = meta.pop('SoftwareTag')
    print("'SoftwareTag' found in metadata for pstdhepsplit module: excluded from input data query")

  res = pstdhepsplit.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.setOutputSE(SE_stdhepsplit)
  wname = process + "_" + str(energy) + "_split"
  wname += additional_name
  pstdhepsplit.setWorkflowName(wname)
  pstdhepsplit.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pstdhepsplit.append(stdhepsplit)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.addFinalization(True, True, True, True)
  descrp = "Splitting stdhep files"
  if additional_name:
    descrp += ", %s" % additional_name
  pstdhepsplit.setDescription(descrp)

  res = pstdhepsplit.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = pstdhepsplit.setProcessIDInFinalPath()
  if not res['OK']:
    print(res['Message'])

  res = pstdhepsplit.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)

  pstdhepsplit.setNbOfTasks(nbtasks_split)
  # As before: get the metadata for this production to input into the next
  meta = pstdhepsplit.getMetadata()
  if tmp_softwaretag_val:  # reinsert SoftwareTag: used in path construction
    meta.update({'SoftwareTag': tmp_softwaretag_val})

  print(" Done With Stdhepsplit", "\n" * 5)

if ild_sim and meta:
  print("################## Creating a prduction for simulation: input meta is")
  pprint.pprint(meta)
  # Define the second production (simulation). Notice the setInputDataQuery call
  pmo = ILDProductionJobDBD()
  pmo.basepath = my_basepath
  pmo.matchToInput = matchToInput_mokka
  pmo.setDryRun(dryrun)
  pmo.setProdPlugin('Standard')
  pmo.setILDConfig(MokkaILDConfig)
  pmo.setEvtClass(my_evtclass)
  pmo.setEvtType(my_evttype)
  pmo.setLogLevel("verbose")
  pmo.setProdType('MCSimulation')
  pmo.setBannedSites(banned_sites)
  pmo.setInputSandbox(input_sand_box)
  # pmo.setDestination(LCG_SITE)
  pmo.setProdPlugin('Limited')

  res = pmo.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmo.setOutputSE(SE_sim)
  wname = process + "_" + str(energy) + "_ild_sim"
  wname += additional_name
  pmo.setWorkflowName(wname)
  pmo.setProdGroup(analysis + "_" + str(energy))
  # Add the application
  res = pmo.append(mo)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmo.addFinalization(True, True, True, True)
  descrp = "%s model" % detectorModel

  if additional_name:
    descrp += ", %s" % additional_name
  pmo.setDescription(descrp)
  res = pmo.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = pmo.setProcessIDInFinalPath()
  if not res['OK']:
    print(res['Message'])

  res = pmo.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmo.setNbOfTasks(nbtasks_sim)
  # As before: get the metadata for this production to input into the next
  meta = pmo.getMetadata()

# Split at slcio level (after sim)
if activesplit and meta:
  print("################## Creating a prduction for sim-slcio split: input meta is")
  pprint.pprint(meta)
  #######################
  # Split the input files.
  psplit = ILDProductionJobDBD()
  psplit.setDryRun(dryrun)
  psplit.setCPUTime(30000)
  psplit.setLogLevel("verbose")
  psplit.setProdType('Split')
  psplit.setInputSandbox(input_sand_box)
  res = psplit.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psplit.setOutputSE(SE_psplit)
  wname = process + "_" + str(energy) + "_split"
  wname += additional_name
  psplit.setWorkflowName(wname)
  psplit.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = psplit.append(split)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psplit.addFinalization(True, True, True, True)
  descrp = "Splitting slcio files"
  if additional_name:
    descrp += ", %s" % additional_name
  psplit.setDescription(descrp)

  res = psplit.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = psplit.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # As before: get the metadata for this production to input into the next
  meta = psplit.getMetadata()

if ild_rec and meta:
  print("################## Creating a production for reconstruction without overlay: input meta is")
  pprint.pprint(meta)
  #######################
  # Define the reconstruction prod
  pma = ILDProductionJobDBD()
  pma.basepath = my_basepath
  pma.setDryRun(dryrun)
  pma.setILDConfig(ILDConfig)
  pma.setLogLevel("verbose")
  pma.setProdType('MCReconstruction')
  pma.setProdPlugin('Limited')

  res = pma.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)

  pma.setOutputSE(SE_rec)
  wname = process + "_" + str(energy) + "_ild_rec"
  wname += additional_name
  pma.setWorkflowName(wname)
  pma.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pma.append(ma)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pma.addFinalization(True, True, True, True)
  descrp = "%s, No overlay" % detectorModel
  if additional_name:
    descrp += ", %s" % additional_name
  pma.setDescription(descrp)

  res = pma.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = pma.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pma.setNbOfTasks(nbtasks_rec)

if ild_rec_ov and meta:
  print("################## Creating a production for reconstruction with overlay: input meta is")
  pprint.pprint(meta)
  #######################
  # Define the reconstruction prod
  pmao = ILDProductionJobDBD()
  pmao.matchToInput = matchToInput_marlin
  pmao.basepath = my_basepath
  pmao.setDryRun(dryrun)
  pmao.setILDConfig(ILDConfig)
  pmao.setEvtClass(my_evtclass)
  # pmao.setUseSoftTagInPath(False)
  pmao.setEvtType(my_evttype)
  pmao.setLogLevel("verbose")
  pmao.setProdType('MCReconstruction_Overlay')
  pmao.setBannedSites(banned_sites)
  pmao.setReconstructionBasePaths(recPath, dstPath)
  pmao.setProdPlugin('Limited')

  res = pmao.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmao.setOutputSE(SE_rec)
  wname = process + "_" + str(energy) + "_ild_rec_overlay"
  wname += additional_name
  pmao.setWorkflowName(wname)
  pmao.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pmao.append(overlay)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # Add the application
  res = pmao.append(mao)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmao.addFinalization(True, True, True, True)
  descrp = "%s, Overlay" % detectorModel

  if additional_name:
    descrp += ", %s" % additional_name
  pmao.setDescription(descrp)
  res = pmao.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = pmao.setProcessIDInFinalPath()
  if not res['OK']:
    print(res['Message'])

  res = pmao.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmao.setNbOfTasks(nbtasks_rec_ov)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the dirac_ilc_add_tasks_to_prod."""


from __future__ import absolute_import
import importlib

import pytest

from mock import MagicMock as Mock

from DIRAC import S_OK, S_ERROR


# pylint: disable=protected-access, invalid-name, missing-docstring, redefined-outer-name
THE_SCRIPT = 'ILCDIRAC.ILCTransformationSystem.scripts.dirac_ilc_add_tasks_to_prod'
theScript = importlib.import_module(THE_SCRIPT)

__RCSID__ = '$Id$'


def transInfoDict(maxTasks, groupSize, plugin):
  """Return the Dictionary with some transformation information."""
  return {'MaxNumberOfTasks': maxTasks,
          'GroupSize': groupSize,
          'Plugin': plugin,
          }


@pytest.fixture
def theParams():
  """Return the Params fixture."""
  return theScript._Params()


@pytest.fixture
def tcClient():
  """Return a TransformationClient Mock."""
  tc = Mock(name='TCClient')
  tc.getTransformation = Mock(return_value=S_OK(transInfoDict(33, 1, 'Standard')))
  tc.setTransformationParameter = Mock(return_value=S_OK())
  return tc


def test_params_init(theParams):
  """Test the Params constructor."""
  assert theParams.prod == 0
  assert theParams.tasks == 0
  assert not theParams.total


def test_params_settters(theParams):
  """Test the Params setters."""
  assert theParams.setProd(12345)['OK']
  assert theParams.prod == 12345
  assert theParams.setProd('555')['OK']
  assert theParams.prod == 555
  assert not theParams.setProd('transName')['OK']

  assert theParams.setNbTasks(123)['OK']
  assert theParams.tasks == 123

  assert theParams.setNbTasks('333')['OK']
  assert theParams.tasks == 333

  assert not theParams.setNbTasks('taskThis')['OK']

  assert theParams.setTotal('dummy')['OK']
  assert theParams.total


def test_params_getMaxTasks(theParams):
  theParams.tasks = 10
  theParams.total = False
  assert theParams.getMaxTasks(10) == 20

  theParams.tasks = 20
  theParams.total = True
  assert theParams.getMaxTasks(10) == 20

  theParams.tasks = 10
  theParams.total = True
  with pytest.raises(theScript._Skip):
    theParams.getMaxTasks(10)


def test_extendStandard_10(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = 10
  theParams.total = False
  trans = transInfoDict(20, 1, 'Standard')
  assert theScript._extendStandard(theParams, tcClient, trans) == 0
  tcClient.setTransformationParameter.assert_called_once_with(111, 'MaxNumberOfTasks', 30)


def test_extendStandard_minus1(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = -1
  theParams.total = False
  trans = transInfoDict(20, 1, 'Standard')
  assert theScript._extendStandard(theParams, tcClient, trans) == 1
  tcClient.setTransformationParameter.assert_not_called()


def test_extendStandard_10_fail(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = 10
  theParams.total = False
  trans = transInfoDict(20, 1, 'Standard')
  tcClient.setTransformationParameter = Mock(return_value=S_ERROR('Failed'))
  assert theScript._extendStandard(theParams, tcClient, trans) == 1
  tcClient.setTransformationParameter.assert_called_once_with(111, 'MaxNumberOfTasks', 30)


def test_extendLimited_10(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = 10
  theParams.total = False
  trans = transInfoDict(20, 1, 'Limited')
  assert theScript._extendLimited(theParams, tcClient, trans) == 0
  tcClient.setTransformationParameter.assert_called_once_with(111, 'MaxNumberOfTasks', 30)


def test_extendLimited_minus1(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = -1
  theParams.total = False
  trans = transInfoDict(20, 1, 'Limited')
  assert theScript._extendLimited(theParams, tcClient, trans) == 0
  tcClient.setTransformationParameter.assert_called_once_with(111, 'MaxNumberOfTasks', -1)


def test_extendLimited_0(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = 0
  theParams.total = False
  trans = transInfoDict(20, 1, 'Limited')
  assert theScript._extendLimited(theParams, tcClient, trans) == 1
  tcClient.setTransformationParameter.assert_not_called()


def test_extendLimited_10_fail(theParams, tcClient):
  theParams.prod = 111
  theParams.tasks = 10
  theParams.total = False
  trans = transInfoDict(20, 1, 'Limited')
  tcClient.setTransformationParameter = Mock(return_value=S_ERROR('Failed'))
  assert theScript._extendLimited(theParams, tcClient, trans) == 1
  tcClient.setTransformationParameter.assert_called_once_with(111, 'MaxNumberOfTasks', 30)


def test_extend_fail(theParams, tcClient):
  posArgs = []
  theScript._getTransformationClient = Mock(return_value=tcClient)
  assert theScript._extend(theParams, posArgs) == 1


def test_extend_fail_pos(theParams, tcClient):
  posArgs = [111, 222, 333]
  theScript._getTransformationClient = Mock(return_value=tcClient)
  assert theScript._extend(theParams, posArgs) == 1


def test_extend_fail_mix(theParams, tcClient):
  posArgs = [111, 222]
  theParams.prod = 444
  theParams.tasks = 555
  theScript._getTransformationClient = Mock(return_value=tcClient)
  assert theScript._extend(theParams, posArgs) == 1


def test_extend_fail_zero(theParams, tcClient):
  posArgs = [0, 0]
  theParams.prod = 0
  theParams.tasks = 0
  theScript._getTransformationClient = Mock(return_value=tcClient)
  assert theScript._extend(theParams, posArgs) == 1


def test_extend_posArgs(theParams, tcClient):
  posArgs = [123, 456]
  theParams.prod = 0
  theParams.tasks = 0
  theScript._getTransformationClient = Mock(return_value=tcClient)
  assert theScript._extend(theParams, posArgs) == 0


def test_extend_standard(theParams, tcClient):
  theParams.prod = 123
  theParams.tasks = 22
  posArgs = []
  theScript._getTransformationClient = Mock(return_value=tcClient)
  assert theScript._extend(theParams, posArgs) == 0
  tcClient.getTransformation.assert_called_once_with(123)


def test_extend_limited(theParams, tcClient):
  theParams.prod = 123
  theParams.tasks = 22
  posArgs = []
  theScript._getTransformationClient = Mock(return_value=tcClient)
  tcClient.getTransformation = Mock(return_value=S_OK(transInfoDict(33, 1, 'Limited')))

  assert theScript._extend(theParams, posArgs) == 0
  tcClient.getTransformation.assert_called_once_with(123)


def test_extend_broad(theParams, tcClient):
  theParams.prod = 123
  theParams.tasks = 22
  posArgs = []
  theScript._getTransformationClient = Mock(return_value=tcClient)
  tcClient.getTransformation = Mock(return_value=S_OK(transInfoDict(33, 1, 'BroadCast')))
  assert theScript._extend(theParams, posArgs) == 1
  tcClient.getTransformation.assert_called_once_with(123)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Utilities for Overlay Files."""
from __future__ import absolute_import
import os
from decimal import Decimal
from math import modf

from DIRAC import gLogger

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id$"


def getOverlayFiles(evttype='gghad'):
  """List the content of the overlay directory given a background type. Called from :func:`~ILCDIRAC.Core.Utilities.PrepareOptionFiles.prepareXMLFile`.

  :param str evttype: type of the background event to find
  :return: list of files contained in the overlay_BKG folder, where BKG can be anything
  """
  localfiles = []
  ovdir = os.path.join(os.getcwd(), "overlayinput_" + evttype)

  if not os.path.exists(ovdir):
    LOG.error('overlay directory does not exists', "overlayinput_" + evttype)
    return localfiles
  listdir = os.listdir(ovdir)
  for item in listdir:
    if item.count('.slcio'):
      localfiles.append(os.path.join(ovdir, item))
  return localfiles


def energyToInt(energy):
  """return energy as integer.

  :param str energy: energy with gev/tev unit
  :return: energy in gev
  :rtype: int
  """
  energy = energy.lower()
  if energy.endswith('tev'):
    return int(float(energy.split('tev')[0]) * 1000)
  if energy.endswith('gev'):
    return int(energy.split('gev')[0])
  raise RuntimeError('Cannot transform %s to energy integer' % energy)


def energyWithUnit(energy):
  """return energy with unit, GeV below 1000, TeV above.

  .. note :: Precision only to the 1 GeV level below 1 TeV, and one digit above 1 TeV

  :param energy: energy
  :type energy: int or float
  :return: energy string with unit
  :rtype: str
  """
  energyString = ''
  if energy < 1000.:
    energyString = "%dGeV" % int(energy)
  elif float(energy / 1000.).is_integer():
    energyString = "%dTeV" % int(energy / 1000.0)
  else:
    energyString = "%1.1fTeV" % float(energy / 1000.0)

  return energyString


def energyWithLowerCaseUnit(energy):
  """return energy with lower case unit, gev below 1000, tev above.

  Same as :func:`energyWithUnit` but lowerCase

  :param energy: energy
  :type energy: int or float
  :return: energy string with unit
  :rtype: str
  """
  return energyWithUnit(energy).lower()


def oldEnergyWithUnit(energy):
  """legacy conversion to energyWithLowerCaseUnit.

  .. deprecated :: v26r0p11
  """
  fracappen = modf(float(energy) / 1000.)
  if fracappen[1] > 0:
    energytouse = "%stev" % (Decimal(str(energy)) / Decimal("1000."))
  else:
    energytouse = "%sgev" % (Decimal(str(energy)))
  if energytouse.count(".0"):
    energytouse = energytouse.replace(".0", "")
  return energytouse



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
'''
:author: sposs
:since: Nov 7, 2013
'''

from __future__ import absolute_import

from DIRAC import S_OK

from ILCDIRAC.Interfaces.API.NewInterface.Application import Application

__RCSID__ = "$Id$"


class LCUtilityApplication(Application):
  """Utility applications."""

  def __init__(self, paramdict=None):
    super(LCUtilityApplication, self).__init__(paramdict)
    # Number of events to process
    self.numberOfEvents = 0
    # Energy to use (duh! again)
    self.energy = 0
    self._importLocation = "ILCDIRAC.Workflow.Modules"

  def setNumberOfEvents(self, numberOfEvents):
    """Set the number of events to process, alias to :func:`setNbEvts`"""
    return self.setNbEvts(numberOfEvents)

  def setEnergy(self, energy):
    """Set the energy to use.

    :param float energy: Energy used in GeV
    """
    if not isinstance(energy, float):
      energy = float(energy)
    self._checkArgs({'energy': float})
    self.energy = energy
    return S_OK()


#### DEPRECATED ################################################################


  def setNbEvts(self, numberOfEvents):
    """Set the number of events to process.

    .. deprecated:: v23r0p0
       use :func:`setNumberOfEvents`

    :param int numberOfEvents: Number of events to process (or generate)
    """
    self._checkArgs({'numberOfEvents': int})
    self.numberOfEvents = numberOfEvents
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the Calibration.py file."""

from __future__ import print_function
from __future__ import absolute_import
import pytest
import os
import shutil
from mock import MagicMock as Mock
from ILCDIRAC.Workflow.Modules.Calibration import Calibration
from DIRAC import S_OK, S_ERROR
from ILCDIRAC.CalibrationSystem.Utilities.functions import searchFilesWithPattern

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.Calibration'


@pytest.fixture
def calib():
  """Create calibration run."""
  calib = Calibration()
  calib.applicationName = "Testing"
  calib.applicationVersion = "vTest"
  calib.debug = True
  calib.platform = 'dummy_platform'
  calib.applicationLog = 'dummy_applicationLog'
  calib.log = Mock()
  calib.detectorModel = 'FCCee_o1_v03'

  calib.SteeringFile = os.path.join(os.environ['ILCDIRAC_BASE_FOLDER'], 'Tests/Files/clicReconstruction.xml')

  yield calib  # provide the fixture value

  dirName = os.path.dirname(os.path.realpath(__file__))
  filesToRemove = searchFilesWithPattern(dirName, 'marlinSteeringFile_*.xml')
  filesToRemove += searchFilesWithPattern(dirName, '%s_%s_Run_*.sh' % (calib.applicationName, calib.applicationVersion))

  potentialFilesToRemove = ['temp.sh', 'localEnv.log']
  for iFile in potentialFilesToRemove:
    fullFileName = os.path.join(dirName, iFile)
    if os.path.exists(fullFileName):
      filesToRemove += [fullFileName]

  for iFile in filesToRemove:
    try:
      os.remove(iFile)
    except EnvironmentError as e:
      print("Failed to delete file: %s" % iFile, str(e))

  del calib
  #  assert False


def test_missingConfigurations(calib, mocker):
  """Test cases when some configuration is missing."""
  calib.platform = None
  res = calib.runIt()
  assert not res['OK']
  assert res['Message'] == 'No ILC platform selected'
  calib.platform = 'dummy'

  calib.applicationLog = None
  res = calib.runIt()
  assert not res['OK']
  assert res['Message'] == 'No Log file provided'
  calib.applicationLog = 'dummy'

  calib.detectorModel = None
  res = calib.runIt()
  assert not res['OK']
  calib.detectorModel = 'dummy'

  mocker.patch.object(calib, 'prepareMARLIN_DLL', new=Mock(return_value=S_ERROR('')))
  res = calib.runIt()
  assert not res['OK']


def test_runScript_properInputArguments(calib, mocker):
  """Test CalibrationRun.runScrip function."""
  mocker.patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK()))
  mocker.patch('os.remove', new=Mock(return_value=True))
  inputxml = 'file1.xml'
  env_script_path = '/dummy/env/path'
  marlin_dll = '/dummy/marlin/dll'
  errorMessageToTest = "steeringfile is missing: %s" % inputxml
  res = calib.runScript(inputxml, env_script_path, marlin_dll)
  assert not res['OK']
  assert res['Message'].lower() == errorMessageToTest.lower()

  mocker.patch('%s.os.path.exists' % MODULE_NAME, return_value=True)
  res = calib.runScript(inputxml, env_script_path, marlin_dll)
  assert res['OK']


paramDictList = [
    {'OK': True, 'Value': {'currentStep': 0, 'currentStage': 1, 'currentPhase': 0, 'parameters': {
        "processor[@name='MyDDMarlinPandora']/parameter[@name='ECalToHadGeVCalibrationEndCap']": 'ECALTOHAD_YYYY'},
        'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 1, 'currentStage': 1,
                           'currentPhase': 1, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 2, 'currentStage': 1,
                           'currentPhase': 2, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 3, 'currentStage': 1,
                           'currentPhase': 3, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 4, 'currentStage': 1,
                           'currentPhase': 4, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 5, 'currentStage': 2,
                           'currentPhase': 5, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 6, 'currentStage': 3,
                           'currentPhase': 0, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 7, 'currentStage': 3,
                           'currentPhase': 1, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 8, 'currentStage': 3,
                           'currentPhase': 2, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 9, 'currentStage': 3,
                           'currentPhase': 3, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 10, 'currentStage': 3,
                           'currentPhase': 4, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 11, 'currentStage': 3,
                           'currentPhase': 4, 'parameters': {}, 'calibrationIsFinished': False}},
    {'OK': True, 'Value': {'currentStep': 12, 'currentStage': 3, 'currentPhase': 4, 'parameters': {},
     'calibrationIsFinished': True}}]


@pytest.fixture
def helper_copyFiles():
  """Copy steering file to local test directory."""
  basePath = os.path.join(os.environ['ILCDIRAC_BASE_FOLDER'], 'Tests', 'Files', 'fcceeConfig')
  dirsToCopy = [os.path.join(basePath, 'CalibrationPandoraSettings'),
                os.path.join(basePath, 'PandoraSettingsFCCee')]
  for iDir in dirsToCopy:
    try:
      shutil.copytree(iDir, iDir.split('/')[-1])
    except EnvironmentError as e:
      print('ERROR Could not copy dir: %s, exception: %s' % (iDir, e))

  yield helper_copyFiles
  if os.path.exists('DISABLE_WATCHDOG_CPU_WALLCLOCK_CHECK'):
    os.remove('DISABLE_WATCHDOG_CPU_WALLCLOCK_CHECK')

  for iDir in dirsToCopy:
    iDir = iDir.split('/')[-1]

    if os.path.exists(iDir):
      try:
        shutil.rmtree(iDir, './')
      except EnvironmentError as e:
        print('ERROR Could not delete dir: %s, exception: %s' % (iDir, e))


def test_runIt_simple(calib, mocker):
  """Test runIt function."""
  mocker.patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK()))
  mocker.patch('%s.os.remove' % MODULE_NAME, return_value=True)
  mocker.patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value={'OK': True, 'Value': 'dummy_EnvScript'}))
  mocker.patch('%s.updateSteeringFile' % MODULE_NAME, new=Mock(return_value={'OK': True, 'Value': None}))
  mocker.patch('%s.os.path.exists' % MODULE_NAME, return_value=True)

  calibClientMock = Mock(name='mock1')
  calibClientMock.requestNewParameters = Mock(side_effect=paramDictList, name='mock3')
  calibClientMock.requestNewPhotonLikelihood.return_value = Mock(return_value='dummy_NewPhotonLikelihood')
  calibClientMock.reportResult.return_value = Mock(return_value=True)
  mocker.patch('%s.CalibrationClient' % MODULE_NAME, new=Mock(return_value=calibClientMock, name='mock2'))

  calib.prepareMARLIN_DLL = Mock(return_value={'OK': True, 'Value': 'dummy_MARLIN_DLL'})
  calib.resolveInputSlcioFilesAndAddToParameterDict = Mock(return_value={'OK': True, 'Value': {}})

  res = calib.runIt()
  assert res['OK']
  print('currentStep: %s' % calib.currentStep)
  assert calib.currentStep == 11


def test_runIt_updatingSteeringFile(helper_copyFiles, calib, mocker):
  """Test runtIt and updateSteeringFile functions."""
  mocker.patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK()))
  mocker.patch('%s.os.remove' % MODULE_NAME, return_value=True)
  mocker.patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value={'OK': True, 'Value': 'dummy_EnvScript'}))
  mocker.patch('%s.os.path.exists' % MODULE_NAME, return_value=True)

  calibClientMock = Mock(name='mock1')
  calibClientMock.requestNewParameters = Mock(side_effect=paramDictList, name='mock3')
  calibClientMock.requestNewPhotonLikelihood.return_value = Mock(return_value='dummy_NewPhotonLikelihood')
  calibClientMock.reportResult.return_value = Mock(return_value=True)
  mocker.patch('%s.CalibrationClient' % MODULE_NAME, new=Mock(return_value=calibClientMock, name='mock2'))

  calib.prepareMARLIN_DLL = Mock(return_value={'OK': True, 'Value': 'dummy_MARLIN_DLL'})
  calib.resolveInputSlcioFilesAndAddToParameterDict = Mock(return_value={'OK': True, 'Value': {}})

  res = calib.runIt()
  assert res['OK']
  print('currentStep: %s' % calib.currentStep)
  assert calib.currentStep == 11


initialParameterDict = {".//global/parameter[@name='MaxRecordNumber']": "10",
                        ".//global/parameter[@name='SkipNEvents']": "1",
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='ECALBarrelTimeWindowMax']": ' 10 ',
                        ".//global/parameter[@name='LCIOInputFiles']": '\n      /afs/cern.ch/work/e/eleogran/public/'
                                                                       'mu_sim/mu_validation_50kevents.slcio\n    ',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='MuonToMipCalibration']":
                        '20703.9',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='ECALEndcapCorrectionFactor']":
                        '1.03245503522',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='ECalToHadGeVCalibrationEndCap']":
                        '1.11490774181',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='HCalToMipCalibration']":
                        '45.6621',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='HCalToEMGeVCalibration']":
                        '1.01776966108',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='ECalToHadGeVCalibrationBarrel']":
                        '1.11490774181',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='HCalToHadGeVCalibration']":
                        '1.00565042407',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='PandoraSettingsXmlFile']":
                        ' PandoraSettingsFCCee/PandoraSettingsDefault.xml ',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='ECalToMipCalibration']":
                        '175.439',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='ECalToEMGeVCalibration']":
                        '1.01776966108',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='HCALEndcapTimeWindowMax']": ' 10 ',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='CalibrHCALBarrel']": '45.9956826061',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='CalibrHCALEndcap']": '46.9252540291',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='CalibrECAL']":
                        '37.5227197175 37.5227197175',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='CalibrHCALOther']": '57.4588011802',
                        ".//parameter[@name='MaxClusterEnergyToApplySoftComp']": 0,
                        ".//processor[@type='InitializeDD4hep']/parameter[@name='DD4hepXMLFile']":
                        '\n      /cvmfs/clicdp.cern.ch/iLCSoft/builds/nightly/x86_64-slc6-gcc62-opt/lcgeo/HEAD/FCCee/'
                        'compact/FCCee_o1_v04/FCCee_o1_v04.xml\n    ',
                        ".//processor[@name='MyDDMarlinPandora_10ns']/parameter[@name='MaxHCalHitHadronicEnergy']":
                        '10000000.',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='ECALEndcapTimeWindowMax']": ' 10 ',
                        ".//processor[@name='MyDDCaloDigi_10ns']/parameter[@name='HCALBarrelTimeWindowMax']": ' 10 ',
                        ".//processor[@name='RootFile']": None}


def test_resolveInputSlcioFilesAndAddToParameterDict(calib, mocker):
  """Test resolveInputSlcioFilesAndAddToParameterDict function."""
  tmpDataDict = {'zuds': (['zuds.slcio'], 1, 24), 'gamma': (['gamma.slcio'], 2, 20),
                 'muon': (['muon.slcio'], 5, 28), 'kaon': (['kaon.slcio'], 8, 31)}
  tmpListOfFiles = ['zuds.slcio', 'gamma.slcio', 'muon.slcio', 'kaon.slcio']
  calibClientMock = Mock(name='mock1')
  calibClientMock.getInputDataDict.return_value = S_OK(tmpDataDict)
  calib.cali = calibClientMock

  calib.currentPhase = 2
  dataFile = 'muon.slcio'
  mocker.patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK([dataFile])))

  calib.currentStage = 1
  res = calib.resolveInputSlcioFilesAndAddToParameterDict(tmpListOfFiles, initialParameterDict)
  print(res)
  assert res['OK']
  assert res['Value'][calib.getKey(res['Value'], 'LCIOInputFiles')] == dataFile
  assert res['Value'][calib.getKey(res['Value'], 'SkipNEvents')] == tmpDataDict['muon'][1]
  assert res['Value'][calib.getKey(res['Value'], 'MaxRecordNumber')] == tmpDataDict['muon'][2]
  assert res['Value'][calib.getKey(res['Value'], 'PandoraSettingsXmlFile')
                      ] == 'PandoraSettingsFCCee/PandoraSettingsDefault.xml'
  assert res['Value'][calib.getKey(res['Value'], 'RootFile')] == 'pfoAnalysis.root'

  calib.currentStage = 2
  res = calib.resolveInputSlcioFilesAndAddToParameterDict(tmpListOfFiles, initialParameterDict)
  print(res)
  assert res['OK']
  assert res['Value'][calib.getKey(res['Value'], 'LCIOInputFiles')] == dataFile
  assert res['Value'][calib.getKey(res['Value'], 'PandoraSettingsXmlFile')
                      ] == 'CalibrationPandoraSettings/PandoraSettingsPhotonTraining.xml'
  assert res['Value'][calib.getKey(res['Value'], 'RootFile')] == 'dummy.root'
  #  assert False



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""unit tests for resolvePathsAndNames module."""

from __future__ import print_function
from __future__ import absolute_import
import unittest
import os
import tempfile
import shutil

from parameterized import parameterized, param

from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename, resolveIFpaths, getProdFilenameFromInput


class ResolvePathsAndNamesTests(unittest.TestCase):
  """Test resolvePathsAndNames."""

  def setUp(self):
    """Make fake files for the test."""
    self.inputfiles = ["toto_gen_12345_123.txt"]
    self.orgDir = os.getcwd()
    self.tempDir = tempfile.mkdtemp(dir=os.getcwd())
    os.chdir(self.tempDir)
    tempDir2 = tempfile.mkdtemp(dir=os.getcwd())
    self.realloc = os.path.join(tempDir2, 'toto_gen_12345_123.txt')
    with open(self.realloc, "w") as inputf:
      inputf.write("fake file")

  def tearDown(self):
    """Remove the fake files."""
    os.chdir(self.orgDir)
    try:
      shutil.rmtree(self.tempDir)
    except OSError:
      print('Failed to remove tempDir')

  def test_getnames(self):
    """test ResolvePathsAndNames getNames..........................................................."""
    res = getProdFilename("toto_gen.stdhep", 12345, 123)
    self.assertEqual(res, 'toto_gen_12345_123.stdhep')

  @parameterized.expand([
      param([]),
      param(['']),
      param(['noSuchFile'], ok=False),
      param(['suchFile'], createFile=True),
      ])
  def test_resolvepaths(self, extraInputs, ok=True, createFile=False):
    """test ResolvePathsAndNames resolvePaths......................................................."""
    extraPaths = []
    if createFile:
      tempfile.mkdtemp(dir=os.getcwd())  # create another empty folder for one path in the function
      for fileName in extraInputs:
        fullPath = os.path.join(self.tempDir, fileName)
        extraPaths.append(fullPath)
        with open(fullPath, 'w') as aFile:
          aFile.write(fullPath)
    res = resolveIFpaths(self.inputfiles + extraInputs)
    assert 'OK' in res
    assert res['OK'] == ok, res
    if ok:
      self.assertTrue('Value' in res, list(res.keys()))
      self.assertEqual(res['Value'], [os.path.abspath(self.realloc)] + extraPaths)

  def test_ildprod_sim(self):
    """test getOridFilenameFromInput Sim ..........................................................."""
    indir = "/ilc/prod/ilc/ild/test/temp1/gensplit/500-TDR_ws/3f/run001/"
    outdir = "/ilc/prod/ilc/test/ild/sim/500-TDR_ws/3f/ILD_o1_v05/v01-19_lcgeo/00001234/000/"
    input_file = indir + "E0500-TDR_ws.Pae_ell.Gwhizard-1.95.eW.pL.I37537.01_002.stdhep"
    outfile_original = outdir + "sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws_sim_400859_4.slcio"
    prodid = 1234
    jobid = 321
    outfile_new = outdir + "sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws.I37537.Pae_ell.eW.pL.n01_002.d_sim_00001234_321.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_rec(self):
    """test getProdFilenameFromInput Rec ..........................................................."""
    indir = "/ilc/prod/ilc/test/ild/sim/500-TDR_ws/3f/ILD_o1_v05/v01-19_lcgeo/00001234/000/"
    outdir = "/ilc/prod/ilc/test/ild/rec/500-TDR_ws/3f/ILD_o1_v05/v01-19_lcgeo/00001235/000/"
    input_file = indir + "sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws.I37540.Pae_ell.eB.pR.n01_002.d_sim_00001234_12.slcio"  # LFN
    outfile_original = outdir + "rv01-19_lcgeo.sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws_rec_1235_321.slcio"
    prodid = 1235
    jobid = 321
    outfile_new = outdir + "rv01-19_lcgeo.sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws.I37540.Pae_ell.eB.pR.n01_002.d_rec_00001235_321.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_rec_mcopt(self):
    """test getProdFilenameFromInput Rec, when sim model and rec model is different.........................."""
    indir = "/ilc/prod/ilc/mc-opt/ild/sim/500-TDR_ws/3f/ILD_l5_v02/v01-19-05/00001234/000/"
    outdir = "/ilc/prod/ilc/mc-opt/ild/rec/500-TDR_ws/3f/ILD_l5_o1_v02/v01-19-05/00001235/000/"
    input_file = indir + "sv01-19-05.mILD_l5_v02.E500-TDR_ws.I37540.Pae_ell.eB.pR.n01_002.d_sim_00001234_62.slcio"  # LFN
    outfile_original = outdir + "rv01-19-05.sv01-19-05.mILD_l5_o1_v02.E500-TDR_ws_rec_1235_421.slcio"
    prodid = 1235
    jobid = 421
    outfile_new = outdir + "rv01-19-05.sv01-19-05.mILD_l5_o1_v02.E500-TDR_ws.I37540.Pae_ell.eB.pR.n01_002.d_rec_00001235_421.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_dbdrec(self):
    """test getProdFilenameFromInput Rec for DBD Sim data .........................................."""
    indir = "/ilc/prod/ilc/mc-dbd/ild/sim/250-TDR_ws/4f_WW_semileptonic/ILD_o1_v05/v01-14-01-p00/"
    outdir = "/ilc/prod/ilc/mc-dbd/ild/250-TDR_ws/4f_WW_semileptonic/ILD_o1_v05/v01-16-02-p01/00001235/000/"
    input_file = indir + "sv01-14-01-p00.mILD_o1_v05.E250-TDR_ws.I106577.P4f_ww_sl.eL.pR-01935.slcio"  # LFN
    outfile_original = outdir + "rv01-16-01-p01.sv01-14-01-p00.mILD_o1_v05.E250-TDR_ws_rec_1235_783.slcio"
    prodid = 1235
    jobid = 783
    outfile_new = outdir + "rv01-16-01-p01.sv01-14-01-p00.mILD_o1_v05.E250-TDR_ws.I106577.P4f_ww_sl.eL.pR.n01935.d_rec_00001235_783.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_dst(self):
    """test getProdFilenameFromInput DST ..........................................................."""
    indir = "/ilc/prod/ilc/test/ild/sim/500-TDR_ws/3f/ILD_o1_v05/v01-19_lcgeo/00001234/000/"
    outdir = "/ilc/prod/ilc/test/ild/dst/500-TDR_ws/3f/ILD_o1_v05/v01-19_lcgeo/00001235/000/"
    input_file = indir + "sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws.I37540.Pae_ell.eB.pR.n01_002.d_sim_00001234_12.slcio"  # LFN
    outfile_original = outdir + "rv01-19_lcgeo.sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws_dst_00001235_321.slcio"
    prodid = 1235
    jobid = 321
    outfile_new = outdir + "rv01-19_lcgeo.sv01-19_lcgeo.mILD_o1_v05.E500-TDR_ws.I37540.Pae_ell.eB.pR.n01_002.d_dst_00001235_321.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_stdhepsplit(self):
    """test getProdFilenameFromInput Generator stdhep  ..........................................................."""
    indir = "/ilc/prod/ilc/mc-dbd/generated/500-TDR_ws/higgs/"
    outdir = "/ilc/prod/ilc/ild/test/temp1/mc-dbd.disk/ild/splitted/500-TDR_ws/higgs_ffh/00400922/000/"
    input_file = indir + "E500-TDR_ws.Pqqh_ww_4q.Gwhizard-1_95.eL.pR.I106730.001.stdhep"
    outfile_original = outdir + "E500-TDR_ws.I106730.Pqqh_ww_4q.eL.pR_gen_400922_1_035.stdhep"
    prodid = 400922
    jobid = 87
    outfile_new = outdir + "E500-TDR_ws.Pqqh_ww_4q.Gwhizard-1_95.eL.pR.I106730.n001_035.d_gen_00400922_87.stdhep"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_stdhepsplit2(self):
    """test getProdFilenameFromInput Generator stdhep 2..........................................................."""
    indir = "/ilc/prod/ilc/mc-dbd/generated/500-TDR_ws/4f/"
    outdir = "/ilc/prod/ilc/ild/test/temp1/mc-dbd.generated/ild/splitted/500-TDR_ws/4f/00008147/000/"
    input_file = indir + "E500-TDR_ws.P4f_zz_sl.Gwhizard-1_95.eL.pR.I250014.031.stdhep"
    outfile_original = outdir + "E500-TDR_ws_gen_8147_1_012.stdhep"
    prodid = 8147
    jobid = 87
    outfile_new = outdir + "E500-TDR_ws.P4f_zz_sl.Gwhizard-1_95.eL.pR.I250014.n031_012.d_gen_00008147_87.stdhep"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_stdhepsplit3(self):
    """test getProdFilenameFromInput Generator stdhep 3..........................................................."""
    indir = "/ilc/prod/ilc/mc-dbd/generated/500-TDR_ws/4f/"
    outdir = "/ilc/prod/ilc/ild/test/temp1/mc-dbd.generated/ild/splitted/500-TDR_ws/4f/00008147/000/"
    input_file = indir + "E500-TDR_ws.P4f_zz_sl.Gwhizard-1_95.eR.pL.I250016.004.stdhep"
    outfile_original = outdir + "E500-TDR_ws_gen_8147_2_065.stdhep"
    prodid = 8147
    jobid = 125
    outfile_new = outdir + "E500-TDR_ws.P4f_zz_sl.Gwhizard-1_95.eR.pL.I250016.n004_065.d_gen_00008147_125.stdhep"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_stdhepsplit_slcio(self):
    """test getProdFilenameFromInput Generator slcio  ..........................................................."""
    indir = "/ilc/prod/ilc/mc-dbd/generated/500-TDR_ws/4f/"
    outdir = "/ilc/prod/ilc/ild/test/temp1/mc-dbd.generated/ild/splitted/500-TDR_ws/4f/00008147/000/"
    input_file = indir + "E500-TDR_ws.P4f_zz_sl.Gwhizard-1_95.eR.pL.I250016.004.slcio"
    outfile_original = outdir + "E500-TDR_ws_gen_8147_2_065.slcio"
    prodid = 8147
    jobid = 125
    outfile_new = outdir + "E500-TDR_ws.P4f_zz_sl.Gwhizard-1_95.eR.pL.I250016.n004_065.d_gen_00008147_125.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)

  def test_ildprod_undef_format(self):
    """test getProdFilenameFromInput Undefined file name ( 1st caharacter is not "E", nor "s" nor "r" .........."""
    indir = "/ilc/user/a/amiyamot/mygen/single_particles/jeans/"
    outdir = "/ilc/user/a/amiyamot/splitted/single_particles/jeans/"
    input_file = indir + "mcparticles_PDG130_MOM20GeV.slcio"
    outfile_original = outdir + "mcparticles_PDG130_MOM20GeV_gen_11520_1_093.slcio"
    prodid = 11520
    jobid = 93
    outfile_new = outdir + "mcparticles_PDG130_MOM20GeV_gen_11520_1_093.slcio"
    out_lfn = getProdFilenameFromInput(input_file, outfile_original, prodid, jobid)
    self.assertEqual(out_lfn, outfile_new)


if __name__ == "__main__":
  SUITE = unittest.defaultTestLoader.loadTestsFromTestCase(ResolvePathsAndNamesTests)
  TESTRESULT = unittest.TextTestRunner(verbosity=2).run(SUITE)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""FCC application is defined by 2 modules which are :

- ILCDIRAC.Interfaces.API.NewInterface.Applications.Fcc
- ILCDIRAC.Workflow.Modules.FccAnalysis (this module)
"""

# standard libraries
from __future__ import absolute_import
import os
import re
import glob

# DIRAC libraries
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getEnvironmentScript
from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC import gLogger, S_ERROR, S_OK

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class FccAnalysis(ModuleBase):
  """FccAnalysis class generates a bash script that will run the FCC application considering FCC module parameters.

  It inherits from ModuleBase class.
  """

  def __init__(self):

    super(FccAnalysis, self).__init__()

    # FCC module parameters
    self.fccExecutable = ''
    self.applicationScript = ''
    self.environmentScript = ''
    self.fccAppIndex = ''
    self.STEP_NUMBER = ''
    self.platform = ''
    self.isGaudiOptionsFileNeeded = False
    self.gaudiOptionsFile = ''
    self.read = False
    self.randomGenerator = {}
    self.RandomSeed = 0

    self.debug = True
    self.enable = True

    # Gaudi log levels
    self.logLevels = ['DEBUG', 'INFO', 'ERROR', 'FATAL']

    # User log level chosen
    self.logLevel = ""

  def runIt(self):
    """Main method called by the Agent. The Application's call must reside here.

       This module consists on creating a bash script calling
       an 'FCC' executable followed by arguments provided by the Fcc module.

    :return: The success or the failure of the execution
        :func:`~DIRAC.Core.Utilities.ReturnValues.S_OK`,
        :func:`~DIRAC.Core.Utilities.ReturnValues.S_ERROR`
    """

    self.result = S_OK()
    if not self.platform:
      errorMessage = 'No ILC platform selected'
      LOG.error(errorMessage)
      self.result = S_ERROR(errorMessage)
    elif not self.applicationLog:
      errorMessage = 'No Log file provided'
      LOG.error(errorMessage)
      self.result = S_ERROR(errorMessage)
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result["Message"])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('%s should not proceed as previous step did not end properly' % self.applicationName)

    if 'IS_PROD' in self.workflow_commons:
      self.RandomSeed = int(
          str(
              int(
                  self.workflow_commons["PRODUCTION_ID"]))
          + str(
              int(
                  self.workflow_commons["JOB_ID"])))
    elif self.jobID:
      self.RandomSeed = int(self.jobID)

    LOG.info("Environment : Environment script look up...")

    # Try to locate environment script in 'dirac.cfg' file
    if not self.getEnvironmentScript():
      errorMessage = (
          "Environment : Environment script look up failed\n"
          "Failed to get environment"
          )
      LOG.error(errorMessage)
      return S_ERROR(errorMessage)

    debugMessage = "Environment : Environment script found at : %s" % self.environmentScript
    LOG.debug(debugMessage)

    LOG.info("Environment : Environment script look up successfull")

    # Some files generated by the application are indexed by 'fccAppIndex'
    self.fccAppIndex = "%s_%s_Step_%s" % (self.applicationName, self.applicationVersion, self.STEP_NUMBER)
    self.applicationScript = os.path.realpath("%s.sh" % self.fccAppIndex)

    getOutputFileFromApp = self.step_commons.get('InputFile', self.InputFile)
    if getOutputFileFromApp:
      if isinstance(getOutputFileFromApp, str):
        self.InputFile = os.path.realpath(getOutputFileFromApp)
      elif isinstance(getOutputFileFromApp, list):
        self.InputFile = [os.path.realpath(inputFile) for inputFile in getOutputFileFromApp]

    # When application.getInputFromApp() method is called
    # SteeringFile is replaced by ouput of previous application
    # if it is FccAnalysis
    if self.InputFile and not self.isGaudiOptionsFileNeeded:
      steeringFiles = [self.InputFile] if isinstance(self.InputFile, str) else self.InputFile
      self.SteeringFile = " ".join(steeringFiles)
      LOG.debug("Application : Configuration file taken from the input file(s) '%s'" % self.SteeringFile)
    # elif self.SteeringFile and not self.SteeringFile.startswith('/cvmfs/'):
    # We uploaded even cvmfs file to sandbox in Fcc module
    elif self.SteeringFile:
      steeringFiles = [os.path.realpath(os.path.basename(self.SteeringFile))]
      self.SteeringFile = steeringFiles[0]

    for steeringFile in steeringFiles:
      if not os.path.exists(steeringFile):
        errorMessage = (
            "Environment : FCC configuration file '%(sfile)s' does not exist,"
            " can not run FCC application" % {'sfile': steeringFile}
            )
        LOG.error(errorMessage)
        return S_ERROR(errorMessage)

    # Set the seed of the application in overwritting Pythia card file
    if "Pythia" in self.randomGenerator:
      for cardFile in self.randomGenerator["Pythia"]:
        absoluteCardFile = os.path.realpath(cardFile)
        content, message = self.readFromFile(absoluteCardFile)

        if not content:
          LOG.error(message)
          return S_ERROR(message)

        LOG.debug(message)

        tempContent = content

        if 'Main:numberOfEvents' in tempContent and self.NumberOfEvents:
          eventSetting = "Main:numberOfEvents = %s" % self.NumberOfEvents
          tempContent = re.sub(r'Main:numberOfEvents *= *\d+', eventSetting, tempContent)
        elif self.NumberOfEvents:
          # add the card line
          eventSetting = ["! N) AUTOMATIC GENERATION OF CODE DONE BY FCC APPLICATION FOR EVENT NUMBER SETTING"]
          eventSetting += ["Main:numberOfEvents = %s         ! number of events to generate" % self.NumberOfEvents]
          tempContent = "%s\n%s\n" % (tempContent, "\n".join(eventSetting))

        if "Random:setSeed" in tempContent and "Random:seed" in tempContent and self.RandomSeed:
          seedSetting = "Random:seed = %d" % self.RandomSeed
          tempContent = re.sub(r'Random:seed *= *\d+', seedSetting, tempContent)
        elif self.RandomSeed:
          seedSetting = ["! N) AUTOMATIC GENERATION OF CODE DONE BY FCC APPLICATION FOR SEED NUMBER SETTING"]
          seedSetting += ["Random:setSeed = on         ! apply user-set seed everytime the Pythia::init is called"]
          seedSetting += ["Random:seed = %d         ! -1=default seed, 0=seed based on time, >0 user seed number" % self.RandomSeed]
          tempContent = "%s\n%s\n" % (tempContent, "\n".join(seedSetting))

        if tempContent != content and not self.writeToFile('w', absoluteCardFile, tempContent):
          errorMessage = "Application : Card file overwitting failed"
          LOG.error(errorMessage)
          return S_ERROR(errorMessage)

    debugMessage = (
        "Application code : Creation of the bash script"
        " to call the application with FCC module parameters..."
        )
    LOG.debug(debugMessage)

    # FCC PHYSICS does not need this file so do not resolve it if it is not given
    # else 'realpath of "" ' will result in cwd.
    if self.isGaudiOptionsFileNeeded:
      if not self.generateGaudiConfFile():
        errorMessage = "Application code : generateGaudiConfFile() failed"
        LOG.error(errorMessage)
        return S_ERROR(errorMessage)

    # Main command
    bashCommand = '%s %s %s' % (self.fccExecutable, self.SteeringFile, self.gaudiOptionsFile)

    if not self.generateBashScript(bashCommand):
      errorMessage = "Application code : Creation of the bash script failed"
      LOG.error(errorMessage)
      return S_ERROR(errorMessage)

    LOG.debug("Application code : Creation of the bash script successfull")

    LOG.debug("Application : Application execution and log file creation...")

    # Call of the application
    call = shellCall(0, self.applicationScript, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)

    if 'OK' in call and not call['OK']:
      errorMessage = "Application : Application execution failed"
      LOG.error(errorMessage)
      return S_ERROR(errorMessage)

    LOG.debug("Application : Application execution successfull")

    if not os.path.exists(self.applicationLog):
      LOG.error("Application : Log file creation failed")
      if not self.ignoreapperrors:
        errorMessage = '%s did not produce the expected log %s' % (self.applicationName, self.applicationLog)
        LOG.error(errorMessage)
        return S_ERROR(errorMessage)

    LOG.debug("Application : Log file creation successfull")

    # Check if root file(s) have been generated
    rootFiles = glob.glob('*.root')

    if not rootFiles:
      LOG.warn("Application : no root files have been generated, was that intentional ?")
    else:
      applicationScriptTimeCreation = os.path.getctime(self.applicationScript)

      # Get root files generated after the script of the application, that means root files generated by the current application !
      # and not by the previous ones
      currentRootFiles = [rootFile for rootFile in rootFiles if os.path.getctime(
          rootFile) > applicationScriptTimeCreation]

      if not currentRootFiles:
        LOG.warn("Application : This application did not generate any root files, was that intentional ?")

      # Renaming of root files at job level already done by DIRAC
      # It can be enabled/disabled by using 'Job.setSplitDoNotAlterOutputFilename'

    return S_OK("Execution of the FCC application successfull")

###############################  FccAnalysis METHODS #############################################
  def generateBashScript(self, command):
    """This function generates a bash script containing the environment setup and the command related to the FCC application.

    :param command: The command of the application
    :type command: list

    :return: The success or the failure of the bash script creation
    :rtype: bool
    """

    # Set environment and execute the application
    shebang = "#!/bin/bash"

    setEnvironmentScript = 'source %s' % self.environmentScript
    bashScriptText = [shebang, setEnvironmentScript, command]

    LOG.debug("Application command : %s" % command)

    # Write the temporary application's script
    LOG.debug("Application code : Bash script creation...")

    if not self.writeToFile('w', self.applicationScript, '\n'.join(bashScriptText) + '\n'):
      LOG.error("Application code : Bash script creation failed")
      return False

    LOG.debug("Application code : Bash script creation successfull")

    LOG.debug("Application file : Bash script rights setting...")

    os.chmod(self.applicationScript, 0o755)

    LOG.debug("Application file : Bash script rights setting successfull")

    return True

  def generateGaudiConfFile(self):
    """Generation of the Gaudi configuration file with the setting of :

      - The number of event
      - The seed
      - The gaudi log level
      - The input file(s) for FCCDataSvc
            ***** TO DO *****
      - The name of the output file

    There are 2 ways for configuring gaudirun.py :
    1) By using gaudirun.py options :
    e.g. ./gaudirun.py --option "ApplicationMgr().EvtMax=10"

    2) By creating a new python script containing the gaudi configuration :
    This script has to be given as a second argument to gaudirun.py
    e.g. ./gaudirun.py geant_pgun_fullsim.py gaudi_options.py
    It then contains the event setting etc...
    We decided to choose the second one.

    :return: The success or the failure of the gaudi option file creation
    :rtype: bool
    """

    gaudiOptions = ["from Configurables import ApplicationMgr"]
    gaudiOptions += ["from Gaudi.Configuration import *"]

    # In putting -1, gaudi read all event of the file given to FCCDataSvc
    if self.NumberOfEvents:
      eventSetting = ["# N) AUTOMATIC GENERATION OF CODE DONE BY FCC APPLICATION FOR EVENT NUMBER SETTING"]
      eventSetting += ["ApplicationMgr().EvtMax=%s" % self.NumberOfEvents]
      gaudiOptions += eventSetting

    if "Gaudi" in self.randomGenerator and self.RandomSeed:

      # GAUDI SEED SETTING
      randomEngineName = "HepRndm::Engine<CLHEP::RanluxEngine>"

      charsToReplace = [':', '<', '>']

      for char in charsToReplace:
        randomEngineName = randomEngineName.replace(char, "_")

      seedSetting = ["# N) AUTOMATIC GENERATION OF CODE DONE BY FCC APPLICATION FOR SEED NUMBER SETTING"]
      seedSetting += ["from Configurables import SimG4Svc, RndmGenSvc"]
      seedSetting += ['from GaudiSvc.GaudiSvcConf import %s' % randomEngineName]
      seedSetting += ["randomEngine = eval('%s')" % randomEngineName]
      seedSetting += ["randomEngine = randomEngine('RndmGenSvc.Engine')"]
      seedSetting += ["randomEngine.Seeds = [%d]  " % self.RandomSeed]

      # GEANT SEED SETTING
      # The seed of Geant is taken from Gaudi
      #seedSetting = ["SimG4Svc().g4PostInitCommands += ['/random/setSeeds %d %d']" % (self.RandomSeed, self.RandomSeed)]

      gaudiOptions += seedSetting

    if self.logLevel.upper() in self.logLevels:
      levelSetting = "ApplicationMgr().OutputLevel=%s" % self.logLevel
      gaudiOptions += [levelSetting]
    else:
      message = (
          "FCCSW specific consistency : Invalid value for the log level\n"
          "Possible values for the log level are :\n%(log)s" % {'log': " ".join(self.logLevels)}
          )
      LOG.error(message)
      return False

    #        ***** TO DO *****
    # outputSetting = ["# N) AUTOMATIC GENERATION OF CODE DONE BY FCC APPLICATION FOR OUTPUT FILENAME SETTING"]
    # outputSetting += ["from Configurables import PodioOutput"]
    # outputSetting += ["out = PodioOutput('output', filename = '%s', OutputLevel = '%s')" % (self.OutputFile,
    # self.logLevel )]
    # outputSetting += ["ApplicationMgr().TopAlg += [out]"]

    #gaudiOptions += outputSetting

    # If it is an application that reads events and there are input data/files to read
    if self.read and (self.InputFile or self.InputData):

      fccswPodioOptions = ["# N) AUTOMATIC GENERATION OF CODE DONE BY FCC APPLICATION FOR INPUT FILE SETTING"]
      fccswPodioOptions += ["from Configurables import FCCDataSvc"]
      fccswPodioOptions += ["import os"]

      dataToRead = self.InputData if self.InputData else self.InputFile

      dataToList = [dataToRead] if isinstance(dataToRead, str) else dataToRead

      fccInputDataSubstitution = ['%s' for data in dataToList]

      if self.InputData:
        # convert /ilc/vo/user/u/username/data1 to /cwd/data1
        fccInputData = ["os.path.realpath(os.path.basename('%s'))" % data
                        for data in dataToList]
      else:
        fccInputData = ["'%s'" % data for data in dataToList]

      # We can provide many input files to FCCDataSvc() like this :
      inputSetting = [
          "podioevent = FCCDataSvc('EventDataSvc', input='%s' %% (%s))" %
          (" ".join(fccInputDataSubstitution), ", ".join(fccInputData))]
      inputSetting += ["ApplicationMgr().ExtSvc += [podioevent]"]
      fccswPodioOptions += inputSetting

      gaudiOptions += fccswPodioOptions

    self.gaudiOptionsFile = '%s_gaudiOptions.py' % self.fccAppIndex

    LOG.debug('FCCSW configuration : Gaudi configuration file creation...')

    return self.writeToFile('w', self.gaudiOptionsFile,
                            "\n".join(gaudiOptions) + '\n')

  def generateScriptOnTheFly(self, sysConfig="", appName="", appVersion=""):
    """Normally, this function has to generate dynamically the FCC environment script but nothing for the moment.

    Called if CVMFS is not available
    (CVMFS should be always available else FCC application can't run).

    :param sysConfig: The platform required by the application
    :type sysConfig: str

    :param appName: The name of the application
    :type appName: str

    :param appVersion: The version of the application
    :type appVersion: str
    """

    # We do not generate the environment script like in MarlinAnalysis etc...
    # Because if we do not have access to cvmfs, we can do nothing.

    #print('%s %s %s' % (sysConfig, appName, appVersion))
    errorMessage = (
        "Environment : Environment script not found\n"
        "for this configuration : %(conf)s, %(name)s, %(version)s\n"
        "Can not generate one dynamically" % {'conf': sysConfig, 'name': appName, 'version': appVersion}
        )

    # Put intentionally in debug level
    LOG.debug(errorMessage)
    return S_ERROR(errorMessage)

  def getEnvironmentScript(self):
    """This function gets environment script path from 'dirac.cfg' file according to the version, application name and platform.

    :return: The success or failure of the environment script creation
      :func:`~DIRAC.Core.Utilities.ReturnValues.S_OK`, :func:`~DIRAC.Core.Utilities.ReturnValues.S_ERROR`
    """

    environmentScript = getEnvironmentScript(self.platform, self.applicationName.lower(),
                                             self.applicationVersion, self.generateScriptOnTheFly)

    if 'OK' in environmentScript and not environmentScript['OK']:
      LOG.error("Environment : 'dirac.cfg' file look up failed")
      return False

    self.environmentScript = environmentScript["Value"]

    return os.path.exists(self.environmentScript)

  def writeToFile(self, operation, fileName, fileText):
    """This function creates a new file and writes the given content into this file.

    :param operation: The operation('w' or 'a') of the writting operation
    :type operation: str

    :param fileName: The name of the file to create
    :type fileName: str

    :param fileText: The content of the file
    :type fileText: str

    :return: success or failure of the write operation
    :rtype: bool
    """

    try:
      # Create file with 'operation' permission
      with open(fileName, operation) as textFile:
        textFile.write(fileText)
    except IOError as e:
      LOG.error("Application : File write operation failed\n%s" % e)
      return False

    LOG.debug("Application : File write operation successfull")
    return True

  @staticmethod
  def readFromFile(fileName):
    """This function reads a file and returns its content.

    :param fileName: The path of the file to read
    :type fileName: str

    :return: The content of the file
    :rtype: str, str
    """

    try:
      with open(fileName, 'r') as fileToRead:
        content = fileToRead.read()
    except IOError as e:
      return None, 'Application : Card file reading failed\n%s' % e

    return content, 'Application : Card file reading successfull'



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test StdHepSplit module."""

from __future__ import absolute_import
import sys
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertInImproved

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.StdHepSplit'

# pylint: disable=protected-access


class StdHepSplitTestCase(unittest.TestCase):
  """Base class for the StdHepSplit test cases."""

  def setUp(self):
    """set up the objects."""
    # Mock out modules that spawn other threads
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': Mock()}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    from ILCDIRAC.Interfaces.API.NewInterface.Applications import StdHepSplit
    self.shs = StdHepSplit({})

  def tearDown(self):
    self.module_patcher.stop()

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.shs._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.shs._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.shs._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.shs._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_checkproductionmeta(self):
    self.shs.numberOfEventsPerFile = 12348
    meta_dict = {'NumberOfEvents': True}
    assertDiracSucceeds(self.shs.checkProductionMetaData(meta_dict), self)
    assertEqualsImproved({'NumberOfEvents': 12348}, meta_dict, self)

  def test_checkproductionmeta_changenothing(self):
    meta_dict = {'myentry': True, 'other_entry': 81943, 'other': 'ae8fj', False: 1}
    assertDiracSucceeds(self.shs.checkProductionMetaData(meta_dict), self)
    assertEqualsImproved({'myentry': True, 'other_entry': 81943, 'other': 'ae8fj', False: 1},
                          meta_dict, self)

  def test_resolvelinkedstepparams(self):
    instance_mock = Mock()
    step_mock = Mock()
    step_mock.getType.return_value = 'abc'
    self.shs._inputappstep = None
    self.shs._jobsteps = ['', '', step_mock]
    self.shs._linkedidx = 2
    assertDiracSucceeds(self.shs._resolveLinkedStepParameters(instance_mock), self)
    instance_mock.setLink.assert_called_once_with('InputFile', 'abc', 'OutputFile')

  def test_resolvelinkedstepparams_nothing_happens(self):
    instance_mock = Mock()
    self.shs._inputappstep = None
    self.shs._jobsteps = None
    self.shs._linkedidx = ['abc']
    assertDiracSucceeds(self.shs._resolveLinkedStepParameters(instance_mock), self)
    self.assertFalse(instance_mock.setLink.called)

  def test_checkconsistency(self):
    self.shs._jobtype = 'notUser'
    self.shs.OutputFile = None
    assertDiracSucceeds(self.shs._checkConsistency(), self)
    assertInImproved({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.shs._listofoutput, self)

  def test_checkconsistency_userjob(self):
    job_mock = Mock()
    job_mock.datatype = 'testDatatype'
    self.shs._job = job_mock
    self.shs._jobtype = 'User'
    self.shs.OutputFile = None
    assertDiracSucceeds(self.shs._checkConsistency(), self)
    self.assertFalse(self.shs.outputFile)
    assertEqualsImproved(self.shs.datatype, 'testDatatype', self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Module to run root executables.

:since:  Apr 29, 2010
:author: Stephane Poss
"""

from __future__ import absolute_import
import os
from DIRAC.Core.Utilities.Subprocess import shellCall
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getEnvironmentScript
from ILCDIRAC.Workflow.Utilities.RootMixin import RootMixin
from DIRAC import S_OK, S_ERROR, gLogger

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class RootExecutableAnalysis(RootMixin, ModuleBase):
  """Run Root executable."""

  def __init__(self):
    super(RootExecutableAnalysis, self).__init__()
    self.enable = True
    self.STEP_NUMBER = ''
    self.applicationName = 'ROOT'
    # from the interface
    self.script = ''
    self.arguments = ''

  def applicationSpecificInputs(self):
    """Resolve all input variables for the module here.

    :return: S_OK()
    """

    if not self.script:
      return S_ERROR("Script no defined")

    return S_OK('Parameters resolved')

  def runIt(self):
    """Called by Agent.

    Execute the following:
      - define the platform
      - check for presence of ROOTSYS variable
    """
    self.result = S_OK()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result['Message'])
      return self.result

    res = getEnvironmentScript(self.platform, "root", self.applicationVersion, self.getRootEnvScript)
    LOG.notice("Got the environment script: %s" % res)
    if not res['OK']:
      LOG.error("Error getting the env script: ", res['Message'])
      return res
    envScriptPath = res['Value']

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('ROOT should not proceed as previous step did not end properly')

    if len(self.script) < 1:
      LOG.error('Executable file not defined, should not happen!')
      return S_ERROR("Executable file not defined")

    self.script = os.path.basename(self.script)

    scriptName = 'Root_%s_Run_%s.sh' % (self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = open(scriptName, 'w')
    script.write('#!/bin/sh \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')

    script.write('source %s\n' % envScriptPath)

    if os.path.exists("./lib"):
      script.write('declare -x LD_LIBRARY_PATH=./lib:$LD_LIBRARY_PATH\n')

    script.write('echo =============================\n')
    script.write('echo LD_LIBRARY_PATH is\n')
    script.write('echo $LD_LIBRARY_PATH | tr ":" "\n"\n')
    script.write('echo =============================\n')
    script.write('echo PATH is\n')
    script.write('echo $PATH | tr ":" "\n"\n')
    script.write('echo =============================\n')
    script.write('env | sort >> localEnv.log\n')
    script.write('echo =============================\n')

    # if the script exists in this folder we add ./ to be sure we execute the local one, otherwise we rely on the PATH
    comm = "%s %s %s\n" % (self.script, self.arguments, self.extraCLIarguments)
    if os.path.exists(self.script):
      script.write("chmod u+x %s\n" % self.script)
      comm = "./" + comm
    LOG.info("Will run %s" % (comm))
    script.write(comm)

    script.write('declare -x appstatus=$?\n')
    # script.write('where\n')
    # script.write('quit\n')
    # script.write('EOF\n')
    script.write('exit $appstatus\n')

    script.close()
    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    os.chmod(scriptName, 0o755)
    comm = 'sh -c "./%s"' % (scriptName)
    self.setApplicationStatus('ROOT %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    #self.result = {'OK':True,'Value':(0,'Disabled Execution','')}
    resultTuple = self.result['Value']
    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('root failed terribly, you are doomed!')
      return S_ERROR('root did not produce the expected log')
    status = resultTuple[0]
    # stdOutput = resultTuple[1]
    # stdError = resultTuple[2]
    LOG.info("Status after the application execution is %s" % str(status))

    return self.finalStatusReport(status)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
'''
:since: Mar 12, 2013
:author: sposs
'''

from __future__ import absolute_import
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import resolveIFpaths

from DIRAC import gLogger, S_OK, S_ERROR
import os
import shutil

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class MoveInFC(ModuleBase):
  """classdocs."""

  def __init__(self):
    """Constructor."""
    super(MoveInFC, self).__init__()
    self.enable = False
    self.STEP_NUMBER = ''
    self.applicationName = 'MoveInFC'
    self.repMan = DataManager()
    self.listoutput = {}
    self.outputpath = ''

  def applicationSpecificInputs(self):
    """Resolve all input variables for the module here.

    :return: S_OK()
    """

    if not len(self.InputFile) and len(self.InputData):
      for files in self.InputData:
        self.InputFile.append(files)

    if 'listoutput' in self.step_commons:
      self.listoutput = self.step_commons['listoutput'][0]

    return S_OK()

  def execute(self):
    """Run the module."""
    result = self.resolveInputVariables()
    if not result['OK']:
      return result
    self.result = S_OK()

    if not self.applicationLog:
      self.result = S_ERROR('No Log file provided')

    if not self.result['OK']:
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('%s should not proceed as previous step did not end properly' % self.applicationName)

    # Now remove the files in the FC
    lfns = self.InputFile

    # Check that all the files are here:
    res = resolveIFpaths(lfns)
    if not res['OK']:
      LOG.error(res['Message'])
      return S_ERROR("Failed to find a file locally")

    # All files are here and available
    paths = res['Value']
    localpaths = []
    for inputfile in paths:
      basename = os.path.basename(inputfile)
      locname = os.path.join(os.getcwd(), basename)
      if not locname == inputfile:
        try:
          shutil.copy(inputfile, locname)
        except shutil.Error:
          LOG.error("Failed to copy file locally, will have to stop")
          return S_ERROR("Failed copy to local directory")
      localpaths.append(locname)
      try:
        os.unlink(inputfile)
      except OSError:
        LOG.warn("Failed to remove initial file, increased \
        disk space usage")

    # all the files are in the run directory

    # get all metadata, ancestor/daughter relations, etc. for all the files

    # Update the listoutput
    if self.listoutput:
      outputlist = []
      for localFile in localpaths:
        item = {}
        item['outputFile'] = localFile
        item['outputPath'] = self.listoutput['outputPath']
        item['outputDataSE'] = self.listoutput['outputDataSE']
        outputlist.append(item)
      if self.enable:
        self.step_commons['listoutput'] = outputlist
      else:
        LOG.info("listoutput would have been ", outputlist)

    # Make sure the path contains / at the end as we are going to
    # concatenate final path and local files
    if not self.outputpath[-1] == '/':
      self.outputpath += "/"

    if 'ProductionOutputData' in self.workflow_commons:
      file_list = ";".join([self.outputpath + name for name in [os.path.basename(fin) for fin in localpaths]])
      if self.enable:
        self.workflow_commons['ProductionOutputData'] = file_list
      else:
        LOG.info("ProductionOutputData would have been", file_list)

    # Now remove them
    if self.enable:
      res = self.repMan.removeFile(lfns, force=True)
      if not res['OK']:
        LOG.error("Failed to remove the files")
        self.setApplicationStatus("Failed to remove the file")
        # return S_ERROR("Failed to remove the files")
    else:
      LOG.info("Would have removed: ", "%s" % str(lfns))

    # Now the files are not on the storage anymore, they exist only locally. We can hope
    # that the job will not be killed between now and the time the UploadOutputData module
    # is called

    return self.finalStatusReport(0)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run TOMATO.

:since: Feb 24, 2011

:author: S. Poss
:author: C. B. Lam
"""

from __future__ import absolute_import
import os

from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder, getEnvironmentScript
from ILCDIRAC.Workflow.Modules.MarlinAnalysis import MarlinAnalysis
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import prepareTomatoSalad
from ILCDIRAC.Core.Utilities.ResolveDependencies import resolveDeps

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class TomatoAnalysis(MarlinAnalysis):
  """Module to run Tomato: the auTOMated Analysis TOol by C.B.

  Lam.
  """

  def __init__(self):
    super(TomatoAnalysis, self).__init__()
    self.applicationName = "Tomato"
    self.collection = ''
    self.InputFile = []

  def applicationSpecificInputs(self):
    """Implement the application specific inputs defined in ModuleBase."""
    if 'Collections' in self.step_commons:
      self.collection = self.step_commons['Collections']

    if 'InputSLCIO' in self.step_commons:
      inputf = self.step_commons["InputSLCIO"]
      if not isinstance(inputf, list):
        inputf = inputf.split(";")
      self.InputFile = inputf

      for files in self.InputData:
        if files.lower().find(".slcio") > -1:
          self.InputFile.append(files)
    return S_OK()

  def runIt(self):
    """Run the module."""
    self.result = S_OK()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error('Failed to resolve input parameters:', self.result['Message'])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('%s should not proceed as previous step did not end properly' % self.applicationName)

    res = getEnvironmentScript(self.platform, "tomato", self.applicationVersion, self.getEnvScript)
    if not res["OK"]:
      LOG.error("Failed to get the env for Tomato:", res["Message"])
      return res
    env_script_path = res["Value"]

    res = self.prepareMARLIN_DLL(env_script_path)
    if not res['OK']:
      LOG.error('Failed building MARLIN_DLL: %s' % res['Message'])
      self.setApplicationStatus('Failed to setup MARLIN_DLL')
      return S_ERROR('Something wrong with software installation')

    marlin_dll = res['Value']

    res = self._getInputFiles()
    if not res['OK']:
      LOG.error(res['Message'])
      return res
    listofslcio = res['Value']

    finalXML = 'tomato.xml'
    res = prepareTomatoSalad(None, finalXML, self.OutputFile, listofslcio, self.collection)
    if not res['OK']:
      LOG.error('Could not prepare the Tomato XML: %s' % res['Message'])
      self.setApplicationStatus('Failed to setup Tomato')
      return S_ERROR('Failed to setup Tomato')

    self.result = self.runMarlin(finalXML, env_script_path, marlin_dll)
    if not self.result['OK']:
      LOG.error('Something wrong during running: %s' % self.result['Message'])
      self.setApplicationStatus('Error during running %s' % self.applicationName)
      return S_ERROR('Failed to run %s' % self.applicationName)

    #self.result = {'OK':True,'Value':(0,'Disabled Execution','')}
    resultTuple = self.result['Value']
    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('%s failed terribly, you are doomed!' % (self.applicationName))
      if not self.ignoreapperrors:
        LOG.error('Missing log file')
        return S_ERROR('%s did not produce the expected log' % (self.applicationName))

    status = resultTuple[0]
    # stdOutput = resultTuple[1]
    # stdError = resultTuple[2]
    LOG.info("Status after the application execution is %s" % str(status))

    return self.finalStatusReport(status)

  def getEnvScript(self, sysconfig, appname, appversion):
    """Called if CVMFS install is not here."""
    res = getSoftwareFolder(sysconfig, appname, appversion)
    if not res['Value']:
      self.setApplicationStatus('Tomato: Could not find neither local area not shared area install')
      return res
    myTomatoDir = res['Value']
    deps = resolveDeps(sysconfig, "tomato", appversion)
    for dep in deps:
      if dep["app"].lower() == 'marlin':
        res = getSoftwareFolder(sysconfig, "marlin", dep["version"])
        if not res['OK']:
          LOG.error('Marlin was not found in software directory')
          return res
        else:
          myMarlinDir = res['Value']
        break

    env_script_name = "TomatoEnv.sh"
    script = open(env_script_name, "w")
    script.write("#!/bin/sh\n")
    script.write('###########################################################\n')
    script.write('# Dynamically generated script to get the Env for Tomato. #\n')
    script.write('###########################################################\n')
    script.write("declare -x PATH=%s/Executable:$PATH\n" % myMarlinDir)
    script.write('declare -x ROOTSYS=%s/ROOT\n' % (myMarlinDir))
    script.write('declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:%s/LDLibs\n' % (myMarlinDir))
    script.write("declare -x LD_LIBRARY_PATH=%s/LDLibs:$LD_LIBRARY_PATH\n" % myTomatoDir)

    script.close()
    return S_OK(os.path.abspath(env_script_name))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run KKMC.

:author: Andrea Stano
:since:  February 03, 2021
"""

from __future__ import absolute_import
import os

from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC import S_OK, S_ERROR, gLogger
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getEnvironmentScript
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class KKMCAnalysis(ModuleBase):
  """Specific Module to run a KKMC job."""

  def __init__(self):
    super(KKMCAnalysis, self).__init__()
    self.enable = True
    self.STEP_NUMBER = ''
    self.result = S_ERROR()
    self.applicationName = 'kkmc'
    self.startFrom = 0
    self.kkmcConfigFile = ''
    self.seedFile = ''
    self.eventType = ''
    # self.eventstring = ['+++ Generating event']
    self.datMan = DataManager()

  def applicationSpecificInputs(self):
    """Resolve all input variables for the module here.

    :return: S_OK()
    """
    if self.workflow_commons.get("IS_PROD"):
      self.OutputFile = getProdFilename(self.OutputFile,
                                        int(self.workflow_commons["PRODUCTION_ID"]),
                                        int(self.workflow_commons["JOB_ID"]),
                                        self.workflow_commons,
                                        )

    return S_OK('Parameters resolved')

  def runIt(self):
    """Called by JobAgent.

    Execute the following:
      - get the environment variables that should have been set during installation
      - prepare the steering file and command line parameters
      - run KKMC on this steering file and catch the exit status

    :rtype: :func:`~DIRAC.Core.Utilities.ReturnValues.S_OK`, :func:`~DIRAC.Core.Utilities.ReturnValues.S_ERROR`
    """
    self.result = S_OK()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result['Message'])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('KKMC should not proceed as previous step did not end properly')

    # get the enviroment script
    res = getEnvironmentScript(
        self.platform,
        self.applicationName,
        self.applicationVersion,
        S_ERROR("No init script provided in CVMFS!"))
    if not res['OK']:
      LOG.error("Could not obtain the environment script: ", res["Message"])
      return res
    envScriptPath = res["Value"]

    CLIArguments = ''

    if self.kkmcConfigFile:
      kkmcSteerName = 'KKMC_%s_Steer_%s.input' % (self.applicationVersion, self.STEP_NUMBER)
      if os.path.exists(kkmcSteerName):
        os.remove(kkmcSteerName)

      kkmcSteer = []
      kkmcSteer.append(self.kkmcConfigFile)

      with open(kkmcSteerName, 'w') as steerFile:
        steerFile.write("\n".join(kkmcSteer))

      CLIArguments += '--config %s ' % kkmcSteerName
    else:
      CLIArguments += '--flavour %s ' % self.eventType
      CLIArguments += '--ecms %s ' % self.energy
      CLIArguments += '--nevts %s ' % self.NumberOfEvents
      CLIArguments += '--outfile %s ' % self.OutputFile

    if self.seedFile:
      seedName = 'KKMC_%s_Seed_%s' % (self.applicationVersion, self.STEP_NUMBER)
      if os.path.exists(seedName):
        os.remove(seedName)
      kkmcSeed = []
      kkmcSeed.append(self.seedFile)

      with open(seedName, 'w') as seedFile:
        seedFile.write("\n".join(kkmcSeed))

      CLIArguments += '--seedfile %s ' % seedName

    scriptName = 'kkmc_%s_Run_%s.sh' % (self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = []
    script.append('#!/bin/bash')
    script.append('#####################################################################')
    script.append('# Dynamically generated script to run a production or analysis job. #')
    script.append('#####################################################################')
    script.append('source %s' % envScriptPath)
    script.append('echo =========')
    script.append('env | sort >> localEnv.log')
    script.append('echo kkmc:`which KKMCee`')
    script.append('echo =========')
    script.append('KKMCee %s' % CLIArguments)
    script.append('declare -x appstatus=$?')
    # FIXME!
    script.append('cp %s events.lhe' % self.OutputFile)
    script.append('exit $appstatus')

    with open(scriptName, 'w') as scriptFile:
      scriptFile.write("\n".join(script))

    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    os.chmod(scriptName, 0o755)
    comm = 'bash "./%s"' % scriptName
    self.setApplicationStatus('KKMC %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    resultTuple = self.result['Value']
    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('%s failed to produce log file' % (self.applicationName))
      if not self.ignoreapperrors:
        return S_ERROR('%s did not produce the expected log %s' % (self.applicationName, self.applicationLog))
    status = resultTuple[0]

    LOG.info("Status after the application execution is %s" % status)

    return self.finalStatusReport(status)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""CalibrationSettings class which defines all required inputs from user."""

from __future__ import print_function
from __future__ import absolute_import
from DIRAC import gLogger
from ILCDIRAC.CalibrationSystem.Utilities.objectFactory import ObjectFactory

__RCSID__ = "$Id$"
LOG = gLogger.getSubLogger(__name__)


class CalibrationSettings(object):
  """Class which defines all required inputs from user."""

  def __init__(self):
    """Initialize settingsDict with default settings."""
    self.settingsDict = {}
    self.settingsDict['digitisationAccuracy'] = 0.05
    self.settingsDict['pandoraPFAAccuracy'] = 0.025
    # FIXME these 2 parameters only for debugging purposes
    self.settingsDict['startStage'] = 1
    self.settingsDict['startPhase'] = 0

    self.settingsDict['numberOfJobs'] = 100
    self.settingsDict['marlinVersion'] = 'ILCSoft-2019-04-01_gcc62'
    # fraction of all jobs must have finished in order for the next step to begin.
    self.settingsDict['fractionOfFinishedJobsNeededToStartNextStep'] = 0.9
    self.settingsDict['disableSoftwareCompensation'] = True
    self.settingsDict['DDPandoraPFANewProcessorName'] = 'MyDDMarlinPandora'
    self.settingsDict['DDCaloDigiName'] = 'MyDDCaloDigi'

    self.settingsDict['nEcalThinLayers'] = 40
    self.settingsDict['nEcalThickLayers'] = 0
    # correction factor which is applied to responce of thick layers of ECAL;
    # if all layer have the same thickness, this factor have to be 1.0
    self.settingsDict['ecalResponseCorrectionForThickLayers'] = 1.0

    # following settings has to be setup for daughter classes
    self.settingsDict['detectorModel'] = None
    self.settingsDict['ecalBarrelCosThetaRange'] = None
    self.settingsDict['ecalEndcapCosThetaRange'] = None
    self.settingsDict['hcalBarrelCosThetaRange'] = None
    self.settingsDict['hcalEndcapCosThetaRange'] = None
    self.settingsDict['nHcalLayers'] = None
    self.settingsDict['steeringFile'] = None
    self.settingsDict['nameOfTheConfigPackage'] = None

    # TODO temporary field in the settings. for testing only
    self.settingsDict['startCalibrationFinished'] = False
    self.settingsDict['stopStage'] = 3
    self.settingsDict['stopPhase'] = 4

    # these settings have no default values... they have to be set up by user
    self.settingsDict['outputSE'] = None
    self.settingsDict['outputPath'] = None
    #  self.settingsDict['pandoraSettingsFile'] = None

  def printSettings(self):
    """Print settings dict in nice format."""
    print('%-59s %s' % ('Settings', 'Value'))
    print('-' * 120)
    for key, value in self.settingsDict.items():
      print('%-59s %s' % (key, value))


class CLDSettings(CalibrationSettings):
  """Child class of CalibrationSettings for CLD detector model."""

  def __init__(self):
    """Initialize."""
    super(CLDSettings, self).__init__()
    self.settingsDict['detectorModel'] = 'FCCee_o1_v04'
    self.settingsDict['ecalBarrelCosThetaRange'] = [0.0, 0.643]
    self.settingsDict['ecalEndcapCosThetaRange'] = [0.766, 0.94]
    self.settingsDict['hcalBarrelCosThetaRange'] = [0.15, 0.485]
    self.settingsDict['hcalEndcapCosThetaRange'] = [0.72, 0.94]
    self.settingsDict['nHcalLayers'] = 44
    # TODO this default will not work... one need to specify full LFN path
    self.settingsDict['steeringFile'] = 'fccReconstruction.xml'
    self.settingsDict['DDPandoraPFANewProcessorName'] = 'MyDDMarlinPandora_10ns'
    self.settingsDict['DDCaloDigiName'] = 'MyDDCaloDigi_10ns'
    self.settingsDict['nameOfTheConfigPackage'] = 'fcceeConfig'
    #  self.settingsDict['pandoraSettingsFile'] = 'PandoraSettingsFCCee/PandoraSettings.xml'


class CLICSettings(CalibrationSettings):
  """Child class of CalibrationSettings for CLIC detector model."""

  def __init__(self):
    """Initialize."""
    super(CLICSettings, self).__init__()
    self.settingsDict['detectorModel'] = 'CLIC_o3_v14'
    self.settingsDict['ecalBarrelCosThetaRange'] = [0.0, 0.65]
    self.settingsDict['ecalEndcapCosThetaRange'] = [0.8, 0.92]
    self.settingsDict['hcalBarrelCosThetaRange'] = [0.2, 0.6]
    self.settingsDict['hcalEndcapCosThetaRange'] = [0.8, 0.9]
    self.settingsDict['nHcalLayers'] = 60
    # TODO this default will not work... one need to specify full LFN path
    self.settingsDict['steeringFile'] = 'clicReconstruction.xml'
    self.settingsDict['nameOfTheConfigPackage'] = 'clicConfig'
    #  self.settingsDict['pandoraSettingsFile'] = 'PandoraSettings/PandoraSettings.xml'


calibrationSettingsFactory = ObjectFactory()
calibrationSettingsFactory.registerBuilder('CLIC', CLICSettings)
calibrationSettingsFactory.registerBuilder('CLD', CLDSettings)


def createCalibrationSettings(detectorModel):
  """Get CalibrationSettings class which corresponds to input detector label and return an instance of the class."""
  calibrationSettingsClass = calibrationSettingsFactory.getClass(detectorModel)
  return calibrationSettingsClass()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""tests for InputFilesUtilities."""
from __future__ import print_function
from __future__ import absolute_import
import unittest
from mock import MagicMock as Mock, patch
from ILCDIRAC.Core.Utilities.InputFilesUtilities import getNumberOfEvents
from Tests.Utilities.GeneralUtils import assertDiracSucceedsWith_equals

from DIRAC import gLogger, S_OK, S_ERROR

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.InputFilesUtilities'


class TestgetNumberOfEvents(unittest.TestCase):
  """tests of getNumberOfEvents."""

  def setUp(self):
    """Make fake files for the test."""
    self.inputfile = "/ilc/prod/ilc/mc-dbd/generated/500-TDR_ws/6f_eeWW/v01-16-p05_500/00005160/000/E500-TDR_ws.I108640.P6f_eexyev.eL.pR_gen_5160_2_065.stdhep"
    self.inputfiles = [self.inputfile, self.inputfile]
    gLogger.setLevel("DEBUG")

  def tearDown(self):
    """Remove the fake files."""
    pass

  def test_getNumberOfEvents(self):
    """test getNumberOfEvents Single File Success..................................................."""
    fcMock = Mock()
    fcMock.getDirectoryUserMetadata = Mock(return_value=S_OK({"NumberOfEvents": 500}))
    fcMock.getFileUserMetadata = Mock(return_value=S_OK({"NumberOfEvents": 500}))

    with patch("%s.FileCatalogClient" % MODULE_NAME, new=Mock(return_value=fcMock)):
      res = getNumberOfEvents(self.inputfiles)
    self.assertEqual(res['Value']['nbevts'], 1000, res.get("Message", ''))

  def test_getNumberOfEvents_2(self):
    """test getNumberOfEvents Multiple File Success................................................."""
    fcMock = Mock()
    fcMock.getDirectoryUserMetadata = Mock(return_value=S_OK({"NumberOfEvents": 500}))
    fcMock.getFileUserMetadata = Mock(return_value=S_OK({"NumberOfEvents": 500}))
    with patch("%s.FileCatalogClient" % MODULE_NAME, new=Mock(return_value=fcMock)):
      res = getNumberOfEvents([self.inputfile])
    self.assertEqual(res['Value']['nbevts'], 500, res.get("Message", ''))

  def test_getNumberOfEvents_Fail(self):
    """test getNumberOfEvents Single File Failure..................................................."""
    fcMock = Mock()
    fcMock.getDirectoryUserMetadata = Mock(return_value=S_ERROR("No Such File"))
    fcMock.getFileUserMetadata = Mock(return_value=S_ERROR("No Such File"))
    with patch("%s.FileCatalogClient" % MODULE_NAME, new=Mock(return_value=fcMock)):
      res = getNumberOfEvents(['/no/such/file'])
    self.assertFalse(res['OK'], res.get("Message", ''))

  def test_getNumberOfEvents_Fail2(self):
    """test getNumberOfEvents Multiple File Failure................................................."""
    fcMock = Mock()
    fcMock.getDirectoryUserMetadata = Mock(return_value=S_ERROR("No Such File"))
    fcMock.getFileUserMetadata = Mock(return_value=S_ERROR("No Such File"))
    with patch("%s.FileCatalogClient" % MODULE_NAME, new=Mock(return_value=fcMock)):
      res = getNumberOfEvents(['/no/such/file', '/no/such2/file2'])
    self.assertFalse(res['OK'], res.get("Message", ''))

  def test_getnumberofevents_othercases(self):
    # Expected behavior:
    # If one file in a directory, get its tags, if Number of events defined go to next entry
    # Else go to directory and check there. if nbevts go to next
    # Else iterate over all files in directory.
    # If numberevents defined nowhere, method fails
    file_meta_dict = {'/unique/dir/file3': S_OK({'Luminosity': '49.2'}),
                       '/one/file/myfile': S_OK({'NumberOfEvents': '14'}),
                       '/other/myfile2': S_OK({'Luminosity': 1489, 'NumberOfEvents': 941.2}),
                       '/a/b/c/Dir1/someFile': S_OK({'NumberOfEvents': '14'}),
                       '/a/b/c/Dir1/other_file': S_OK({'NumberOfEvents': '14'}),
                       '/a/b/c/Dir1/dontforget_me': S_OK({'NumberOfEvents': '14'})}
    directory_meta_dict = {'/a/b/c/Dir1': S_OK({'Luminosity': 84.1, 'evttype': 'testEvt'}),
                            '/unique/dir': S_OK({'NumberOfEvents': 814, 'Luminosity': None}),
                            '/other': S_OK({'NumberOfEvents': None, 'Luminosity': None}),
                            '/one/file': S_OK({})}
    fcMock = Mock()
    fcMock.getDirectoryUserMetadata = Mock(side_effect=lambda path: directory_meta_dict[path])
    fcMock.getFileUserMetadata = Mock(side_effect=lambda filename: file_meta_dict[filename])
    with patch("%s.FileCatalogClient" % MODULE_NAME, new=Mock(return_value=fcMock)):
      assertDiracSucceedsWith_equals(getNumberOfEvents(['/a/b/c/Dir1/someFile', '', '/a/b/c/Dir1/other_file',
                                                           '/unique/dir/file3', '', '', '',
                                                           '/a/b/c/Dir1/dontforget_me', '/one/file/myfile',
                                                           '/other/myfile2']),
                                      {'AdditionalMeta': {'evttype': 'testEvt'}, 'EvtType': '',
                                        'lumi': 1790.5, 'nbevts': 1811}, self)

  def test_getnumberofevents_rarecase(self):
    file_meta_dict = {'/a/b/c/Dir1/someFile': S_OK({'NumberOfEvents': '14'}),
                       '/a/b/c/Dir1/other_file': S_OK({'Luminosity': '14.5'}),
                       '/a/b/c/Dir1/dontforget_me': S_OK({'NumberOfEvents': '14'})}
    directory_meta_dict = {'/a/b/c/Dir1': S_ERROR({'Luminosity': 84.25, 'evttype': 'testEvt'}),
                            '/unique/dir': S_ERROR({'NumberOfEvents': 814, 'Luminosity': None}),
                            '/other': S_ERROR({'NumberOfEvents': None, 'Luminosity': None}),
                            '/one/file': S_OK({})}
    fcMock = Mock()
    fcMock.getDirectoryUserMetadata = Mock(side_effect=lambda path: directory_meta_dict[path])
    fcMock.getFileUserMetadata = Mock(side_effect=lambda filename: file_meta_dict[filename])
    with patch("%s.FileCatalogClient" % MODULE_NAME, new=Mock(return_value=fcMock)):
      assertDiracSucceedsWith_equals(getNumberOfEvents(['/a/b/c/Dir1/someFile', '', '/a/b/c/Dir1/other_file',
                                                           '', '', '', '/a/b/c/Dir1/dontforget_me']),
                                      {'AdditionalMeta': {}, 'EvtType': '', 'lumi': 14.5, 'nbevts': 28},
                                      self)


if __name__ == "__main__":
  SUITE = unittest.defaultTestLoader.loadTestsFromTestCase(TestgetNumberOfEvents)
  TESTRESULT = unittest.TextTestRunner(verbosity=2).run(SUITE)
  print(TESTRESULT)



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test StdhepCut module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from parameterized import parameterized

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import StdhepCut
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.StdhepCut'

# pylint: disable=protected-access


class StdhepCutTestCase(unittest.TestCase):
  """Base class for the StdhepCut test cases."""

  def setUp(self):
    """set up the objects."""
    self.shc = StdhepCut({})

  @parameterized.expand([('MaxNbEvts', 'maxNumberOfEvents', 1234, 'str'),
                         ('NbEvtsPerFile', 'numberOfEventsPerFile', 333, 'args'),
                         ('FileMask', 'fileMask', '*.stdhep', ['args']),
                         ('SelectionEfficiency', 'selectionEfficiency', 0.5, 'asd'),
                         ('InlineCuts', 'inlineCuts', 'someVal_LT 100', None),
                         ('StoreFile', 'storeFile', False, 'False'),
                        ])
  def testSetter(self, funcName, attribute, value, failValue):
    """Parameterized test for setter functions."""
    self.assertFalse(self.shc._errorDict)
    getattr(self.shc, 'set%s' % funcName)(value)
    assertEqualsImproved(getattr(self.shc, attribute), value, self)
    if failValue is None:
      return
    self.assertFalse(self.shc._errorDict)
    getattr(self.shc, 'set%s' % funcName)(failValue)
    self.assertIn('_checkArgs', self.shc._errorDict)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.shc._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.shc._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.shc._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.shc._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_resolvelinkedparams(self):
    step_mock = Mock()
    input_mock = Mock()
    input_mock.getType.return_value = {'abc': False}
    self.shc._linkedidx = 3
    self.shc._jobsteps = [None, None, None, input_mock]
    assertDiracSucceeds(self.shc._resolveLinkedStepParameters(step_mock), self)
    step_mock.setLink.assert_called_once_with('InputFile', {'abc': False}, 'OutputFile')

  def test_resolvelinkedparams_noinputstep(self):
    self.shc._linkedidx = None
    self.shc._inputappstep = []
    assertDiracSucceeds(self.shc._resolveLinkedStepParameters(None), self)

  def test_checkfinalconsistency_noevents(self):
    self.shc.numberOfEvents = 0
    assertDiracFailsWith(self.shc._checkFinalConsistency(), 'specify the number of events', self)

  def test_checkfinalconsistency_toofewevts(self):
    self.shc.numberOfEvents = 418
    self.shc.selectionEfficiency = 4
    self.shc.maxNumberOfEvents = 1000000
    assertDiracFailsWith(self.shc._checkFinalConsistency(), "don't generate enough events", self)

  def test_checkfinalconsistency(self):
    self.shc.numberOfEvents = 418
    self.shc.selectionEfficiency = 4
    self.shc.maxNumberOfEvents = 10
    assertDiracSucceeds(self.shc._checkFinalConsistency(), self)

  def test_checkconsistency(self):
    self.shc.steeringFile = 'abc.xml'
    self.shc.inlineCuts = None
    self.shc.maxNumberOfEvents = 14
    self.shc.selectionEfficiency = 183
    self.shc._jobtype = 'User'
    assertDiracSucceeds(self.shc._checkConsistency(), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.shc._listofoutput)
    self.assertNotIn('nbevts', self.shc.prodparameters)
    self.assertNotIn('Process', self.shc.prodparameters)

  def test_checkconsistency_nocuts(self):
    self.shc.steeringFile = None
    self.shc.inlineCuts = None
    assertDiracFailsWith(self.shc._checkConsistency(), 'cuts not specified', self)

  def test_checkconsistency_nomaxnbevts(self):
    self.shc.steeringFile = 'abc.xml'
    self.shc.inlineCuts = 'cuts.pdf'
    self.shc.maxNumberOfEvents = None
    assertDiracFailsWith(self.shc._checkConsistency(), 'did not specify how many events', self)

  def test_checkconsistency_noefficiency(self):
    self.shc.steeringFile = 'abc.xml'
    self.shc.inlineCuts = 'cuts.pdf'
    self.shc.maxNumberOfEvents = 14
    self.shc.selectionEfficiency = None
    assertDiracFailsWith(self.shc._checkConsistency(),
                          'need to know the selection efficiency of your cuts', self)

  def test_checkconsistency_nouserjob(self):
    self.shc.steeringFile = 'abc.xml'
    self.shc.inlineCuts = None
    self.shc.maxNumberOfEvents = 14
    self.shc.selectionEfficiency = 38
    self.shc._jobtype = 'notUser'
    assertDiracSucceeds(self.shc._checkConsistency(), self)
    self.assertIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                     'outputDataSE': '@{OutputSE}'}, self.shc._listofoutput)
    self.assertIn('nbevts_kept', self.shc.prodparameters)
    self.assertIn('cut_file', self.shc.prodparameters)



#!/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Create a production to replicate files from one storage element to another.

Example::

  dirac-ilc-replication-transformation <prodID> <TargetSEs> <SourceSEs> {GEN,SIM,REC,DST} -NExtraName

Options:
   -N, --Extraname string      String to append to transformation name in case one already exists with that name
   -R, --GroupName <value>     TransformationGroup Name, by itself the group of the prodID
   -G, --GroupSize <value>     Number of Files per transformation task
   -x, --Enable                Enable the transformation creation, otherwise dry-run

:since:  May 18, 2015
:author: A. Sailer
"""
from __future__ import absolute_import
from pprint import pformat

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import gLogger, exit as dexit
from DIRAC.TransformationSystem.Utilities.ReplicationTransformation import createDataTransformation

from ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters import Params as _Params, getTransformationGroup

__RCSID__ = "$Id$"

LOG = gLogger.getSubLogger("ReplTrans")


def registerSwitches(script):
  """register additional switches for replication transformation."""
  script.setUsageMessage("""%s <prodID> <TargetSEs> <SourceSEs> {GEN,SIM,REC,DST} -NExtraName""" % script.scriptName)


@DIRACScript()
def main():
  """reads command line parameters, makes check and creates replication transformation."""
  clip = _Params()
  clip.groupSize = 10
  clip.registerSwitches(Script)
  registerSwitches(Script)
  Script.parseCommandLine()
  if not clip.checkSettings(Script)['OK']:
    LOG.error("ERROR: Missing settings")
    return 1
  for prodID in clip.metaValues:
    tGroup = getTransformationGroup(prodID, clip.groupName)
    parDict = dict(flavour=clip.flavour,
                   targetSE=clip.targetSE,
                   sourceSE=clip.sourceSE,
                   metaKey=clip.metaKey,
                   metaValue=prodID,
                   extraData={'Datatype': clip.datatype},
                   extraname=clip.extraname,
                   plugin=clip.plugin,
                   groupSize=clip.groupSize,
                   tGroup=tGroup,
                   enable=clip.enable,
                   )
    LOG.notice('Parameters: %s' % pformat(parDict))
    resCreate = createDataTransformation(**parDict)
    if not resCreate['OK']:
      LOG.error('Failed to create the transformation', resCreate['Message'])
      return 1

  return 0


if __name__ == '__main__':
  dexit(main)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Tests for Utilities functions."""

from __future__ import absolute_import
import pytest

from ILCDIRAC.Core.Utilities.Utilities import toInt, listify, lowerFirst, canConvert


@pytest.mark.parametrize("number, expected, cond",
                         [("1", 1, None),
                          ("1.2", False, None),
                          (1.2, 1, None),
                          ("-1", -1, None),
                          (None, None, None),
                          ("a", False, None),
                          ("-1", False, lambda x: x > 0),
                          ("12", 12, lambda x: x > 0),
                          ])
def test_toint(number, expected, cond):
  """Testing the to int function."""
  assert toInt(number, cond=cond) == expected


@pytest.mark.parametrize("string, cast, expected",
                         [("1", None, ['1']),
                          ("1,3", None, ['1', '3']),
                          ("1,3,,,", int, [1, 3]),
                          ("0, 1,3", int, [0, 1, 3]),
                          ("  foo  , bar  ", None, ['foo', 'bar']),
                          ([1, 3, 4], None, [1, 3, 4]),
                          ])
def test_listify(string, cast, expected):
  """Testing the to int function."""
  assert listify(string, cast) == expected


@pytest.mark.parametrize('string, expected',
                         [('1', '1'),
                          ('SOMETHING', 'sOMETHING'),
                          ('something', 'something'),
                          ('CamelCase', 'camelCase'),
                          ])
def test_lowerFirst(string, expected):
  """Testing the lowerFirst function."""
  assert lowerFirst(string) == expected


@pytest.mark.parametrize('string, cast, expected',
                         [('5', int, True),
                           ('Five', int, False),
                           ('5.', int, False),
                           ('5.0', float, True),
                           ('5.0e10', float, True),
                           ('fff5.0f', float, False),
                           ])
def test_canConvert(string, cast, expected):
  """Testing the canConvert function."""
  assert canConvert(string, cast) == expected
